from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.cloudformation.checks.resource.base_resource_value_check import BaseResourceValueCheck


class RDSMultiAZEnabled(BaseResourceValueCheck):
    def __init__(self) -> None:
        name = "Ensure that RDS instances have Multi-AZ enabled"
        id = "CKV_AWS_157"
        supported_resources = ("AWS::RDS::DBInstance",)
        categories = (CheckCategories.NETWORKING,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
        # Aurora is replicated across all AZs and doesn't require MultiAZ to be set
        # https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-rds-dbinstance.html#cfn-rds-dbinstance-multiaz
        if 'Properties' in conf.keys():
            if 'Engine' in conf['Properties'].keys():
                if 'aurora' in conf['Properties']['Engine']:
                    return CheckResult.UNKNOWN
        # Database is not Aurora; Use base class implementation
        return super().scan_resource_conf(conf)

    def get_inspected_key(self) -> str:
        return 'Properties/MultiAZ'


check = RDSMultiAZEnabled()



from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck


class AppServiceFTPSState(BaseResourceValueCheck):
    def __init__(self):
        name = "Ensure FTP deployments are disabled"
        id = "CKV_AZURE_78"
        supported_resources = ('azurerm_app_service', 'azurerm_linux_web_app', 'azurerm_windows_web_app')
        categories = (CheckCategories.APPLICATION_SECURITY,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self):
        return "site_config/0/ftps_state"

    def get_expected_value(self):
        return "Disabled"

    def get_expected_values(self):
        return ["Disabled", "FtpsOnly"]


check = AppServiceFTPSState()



from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck
from checkov.common.models.enums import CheckCategories


class RDSClusterCopyTags(BaseResourceValueCheck):
    def __init__(self) -> None:
        """
        NIST.800-53.r5 CA-9(1), NIST.800-53.r5 CM-2, NIST.800-53.r5 CM-2(2)
        RDS DB clusters should be configured to copy tags to snapshots
        """
        name = "Ensure RDS cluster configured to copy tags to snapshots"
        id = "CKV_AWS_313"
        supported_resources = ("aws_rds_cluster",)
        categories = (CheckCategories.GENERAL_SECURITY,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        return "copy_tags_to_snapshot"


check = RDSClusterCopyTags()



from __future__ import annotations

from typing import Any

from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.cloudformation.checks.resource.base_resource_check import BaseResourceCheck


class LambdaEnvironmentEncryptionSettings(BaseResourceCheck):
    def __init__(self) -> None:
        name = "Check encryption settings for Lambda environmental variable"
        id = "CKV_AWS_173"
        supported_resources = ("AWS::Lambda::Function", "AWS::Serverless::Function")
        categories = (CheckCategories.ENCRYPTION,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: dict[str, Any]) -> CheckResult:
        properties = conf.get("Properties")
        if properties is not None:
            env = properties.get("Environment")
            if env is not None:
                if not isinstance(env, dict):
                    return CheckResult.UNKNOWN
                elif env.get("Variables") and not properties.get("KmsKeyArn"):
                    return CheckResult.FAILED
        return CheckResult.PASSED

    def get_evaluated_keys(self) -> list[str]:
        return ["Properties/KmsKeyArn"]


check = LambdaEnvironmentEncryptionSettings()



from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_negative_value_check import BaseResourceNegativeValueCheck


class NKSPublicAccess(BaseResourceNegativeValueCheck):
    def __init__(self) -> None:
        name = "Ensure Naver Kubernetes Service public endpoint disabled"
        id = "CKV_NCP_19"
        supported_resources = ("ncloud_nks_cluster",)
        categories = (CheckCategories.KUBERNETES,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self):
        return 'public_network'

    def get_forbidden_values(self):
        return [True]


check = NKSPublicAccess()



from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck


class K8sNodePoolAutoRepair(BaseResourceValueCheck):
    def __init__(self) -> None:
        name = "Ensure K8s nodepools are set to auto repair"
        id = "CKV_ALI_31"
        supported_resources = ("alicloud_cs_kubernetes_node_pool",)
        categories = (CheckCategories.KUBERNETES,)
        super().__init__(
            name=name,
            id=id,
            categories=categories,
            supported_resources=supported_resources,
            missing_block_result=CheckResult.FAILED,
        )

    def get_inspected_key(self) -> str:
        return "management/auto_repair"


check = K8sNodePoolAutoRepair()



from __future__ import annotations

import logging
import fnmatch
from collections import defaultdict
from collections.abc import Iterable
from typing import Any, Set, Optional, Union, List, TYPE_CHECKING, Dict, DefaultDict, cast
import re

from checkov.common.bridgecrew.check_type import CheckType
from checkov.common.secrets.consts import ValidationStatus

from checkov.common.bridgecrew.code_categories import CodeCategoryMapping, CodeCategoryConfiguration, CodeCategoryType
from checkov.common.bridgecrew.severities import Severity, Severities
from checkov.common.util.consts import DEFAULT_EXTERNAL_MODULES_DIR
from checkov.common.util.type_forcers import convert_csv_string_arg_to_list
from checkov.common.util.str_utils import convert_to_seconds

if TYPE_CHECKING:
    from checkov.common.checks.base_check import BaseCheck
    from checkov.common.graph.checks_infra.base_check import BaseGraphCheck


class RunnerFilter(object):
    # NOTE: This needs to be static because different filters may be used at load time versus runtime
    #       (see note in BaseCheckRegistery.register). The concept of which checks are external is
    #       logically a "static" concept anyway, so this makes logical sense.
    __EXTERNAL_CHECK_IDS: Set[str] = set()

    def __init__(
            self,
            framework: Optional[List[str]] = None,
            checks: Union[str, List[str], None] = None,
            skip_checks: Union[str, List[str], None] = None,
            include_all_checkov_policies: bool = True,
            download_external_modules: bool = False,
            external_modules_download_path: str = DEFAULT_EXTERNAL_MODULES_DIR,
            evaluate_variables: bool = True,
            runners: Optional[List[str]] = None,
            skip_framework: Optional[List[str]] = None,
            excluded_paths: Optional[List[str]] = None,
            all_external: bool = False,
            var_files: Optional[List[str]] = None,
            skip_cve_package: Optional[List[str]] = None,
            use_enforcement_rules: bool = False,
            filtered_policy_ids: Optional[List[str]] = None,
            show_progress_bar: Optional[bool] = True,
            run_image_referencer: bool = False,
            enable_secret_scan_all_files: bool = False,
            block_list_secret_scan: Optional[List[str]] = None,
            deep_analysis: bool = False,
            repo_root_for_plan_enrichment: Optional[List[str]] = None,
            resource_attr_to_omit: Optional[Dict[str, Set[str]]] = None,
            enable_git_history_secret_scan: bool = False,
            git_history_timeout: str = '12h',
            git_history_last_commit_scanned: Optional[str] = None  # currently not exposed by a CLI flag
    ) -> None:

        checks = convert_csv_string_arg_to_list(checks)
        skip_checks = convert_csv_string_arg_to_list(skip_checks)

        self.skip_invalid_secrets = skip_checks and any(skip_check.capitalize() == ValidationStatus.INVALID.value
                                                        for skip_check in skip_checks)

        self.use_enforcement_rules = use_enforcement_rules
        self.enforcement_rule_configs: Dict[str, Severity | Dict[CodeCategoryType, Severity]] = {}

        # we will store the lowest value severity we find in checks, and the highest value we find in skip-checks
        # so the logic is "run all checks >= severity" and/or "skip all checks <= severity"
        self.check_threshold = None
        self.skip_check_threshold = None
        self.checks = []
        self.bc_cloned_checks: dict[str, list[dict[str, Any]]] = defaultdict(list)
        self.skip_checks = []
        self.skip_checks_regex_patterns = defaultdict(list)
        self.show_progress_bar = show_progress_bar

        # split out check/skip thresholds so we can access them easily later
        for val in (checks or []):
            if val.upper() in Severities:
                val = val.upper()
                if not self.check_threshold or self.check_threshold.level > Severities[val].level:
                    self.check_threshold = Severities[val]
            else:
                self.checks.append(val)
        # Get regex patterns to split checks and remove it from skip checks:
        updated_skip_checks = set(skip_checks)
        for val in (skip_checks or []):
            splitted_check = val.split(":")
            # In case it's not expected pattern
            if len(splitted_check) != 2:
                continue
            self.skip_checks_regex_patterns[splitted_check[0]].append(splitted_check[1])
            updated_skip_checks -= {val}

        skip_checks = list(updated_skip_checks)
        for val in (skip_checks or []):
            if val.upper() in Severities:
                val = val.upper()
                if not self.skip_check_threshold or self.skip_check_threshold.level < Severities[val].level:
                    self.skip_check_threshold = Severities[val]
            else:
                self.skip_checks.append(val)

        self.include_all_checkov_policies = include_all_checkov_policies

        self.framework: "Iterable[str]" = framework if framework else ["all"]
        if skip_framework:
            if "all" in self.framework:
                if runners is None:
                    runners = []

                self.framework = set(runners) - set(skip_framework)
            else:
                self.framework = set(self.framework) - set(skip_framework)
        logging.debug(f"Resultant set of frameworks (removing skipped frameworks): {','.join(self.framework)}")

        self.download_external_modules = download_external_modules
        self.external_modules_download_path = external_modules_download_path
        self.evaluate_variables = evaluate_variables
        self.excluded_paths = excluded_paths or []
        self.all_external = all_external
        self.var_files = var_files
        self.skip_cve_package = skip_cve_package
        self.filtered_policy_ids = filtered_policy_ids or []
        self.run_image_referencer = run_image_referencer
        self.enable_secret_scan_all_files = enable_secret_scan_all_files
        self.block_list_secret_scan = block_list_secret_scan
        self.suppressed_policies: List[str] = []
        self.deep_analysis = deep_analysis
        self.repo_root_for_plan_enrichment = repo_root_for_plan_enrichment
        self.resource_attr_to_omit: DefaultDict[str, Set[str]] = RunnerFilter._load_resource_attr_to_omit(
            resource_attr_to_omit
        )
        self.enable_git_history_secret_scan: bool = enable_git_history_secret_scan
        if self.enable_git_history_secret_scan:
            self.git_history_timeout = convert_to_seconds(git_history_timeout)
            self.framework = [CheckType.SECRETS]
            logging.debug("Scan secrets history was enabled ignoring other frameworks")
            self.git_history_last_commit_scanned = git_history_last_commit_scanned

    @staticmethod
    def _load_resource_attr_to_omit(resource_attr_to_omit_input: Optional[Dict[str, Set[str]]]) -> DefaultDict[str, Set[str]]:
        resource_attributes_to_omit: DefaultDict[str, Set[str]] = defaultdict(set)
        # In order to create new object (and not a reference to the given one)
        if resource_attr_to_omit_input:
            resource_attributes_to_omit.update(resource_attr_to_omit_input)
        return resource_attributes_to_omit

    def apply_enforcement_rules(self, enforcement_rule_configs: Dict[str, CodeCategoryConfiguration]) -> None:
        self.enforcement_rule_configs = {}
        for report_type, code_category in CodeCategoryMapping.items():
            if isinstance(code_category, list):
                self.enforcement_rule_configs[report_type] = {c: enforcement_rule_configs.get(c).soft_fail_threshold for c in code_category}  # type:ignore[union-attr] # will not be None
            else:
                config = enforcement_rule_configs.get(code_category)
                if not config:
                    raise Exception(f'Could not find an enforcement rule config for category {code_category} (runner: {report_type})')
                self.enforcement_rule_configs[report_type] = config.soft_fail_threshold

    def extract_enforcement_rule_threshold(self, check_id: str, report_type: str) -> Severity:
        if 'sca_' in report_type and '_LIC_' in check_id:
            return cast("dict[CodeCategoryType, Severity]", self.enforcement_rule_configs[report_type])[CodeCategoryType.LICENSES]
        elif 'sca_' in report_type:  # vulnerability
            return cast("dict[CodeCategoryType, Severity]", self.enforcement_rule_configs[report_type])[CodeCategoryType.VULNERABILITIES]
        else:
            return cast(Severity, self.enforcement_rule_configs[report_type])

    def should_run_check(
            self,
            check: BaseCheck | BaseGraphCheck | None = None,
            check_id: str | None = None,
            bc_check_id: str | None = None,
            severity: Severity | None = None,
            report_type: str | None = None,
            file_origin_paths: List[str] | None = None,
            root_folder: str | None = None
    ) -> bool:
        if check:
            check_id = check.id
            bc_check_id = check.bc_id
            severity = check.severity

        assert check_id is not None  # nosec (for mypy (and then for bandit))

        check_threshold: Optional[Severity]
        skip_check_threshold: Optional[Severity]

        # apply enforcement rules if specified, but let --check/--skip-check with a severity take priority
        if self.use_enforcement_rules and report_type:
            if not self.check_threshold and not self.skip_check_threshold:
                check_threshold = self.extract_enforcement_rule_threshold(check_id, report_type)
                skip_check_threshold = None
            else:
                check_threshold = self.check_threshold
                skip_check_threshold = self.skip_check_threshold
        else:
            if self.use_enforcement_rules:
                # this is a warning for us (but there is nothing the user can do about it)
                logging.debug(f'Use enforcement rules is true, but check {check_id} was not passed to the runner filter with a report type')
            check_threshold = self.check_threshold
            skip_check_threshold = self.skip_check_threshold

        run_severity = severity and check_threshold and severity.level >= check_threshold.level
        explicit_run = self.checks and self.check_matches(check_id, bc_check_id, self.checks)
        implicit_run = not self.checks and not check_threshold
        is_external = RunnerFilter.is_external_check(check_id)
        is_policy_filtered = self.is_policy_filtered(check_id)
        # True if this check is present in the allow list, or if there is no allow list
        # this is not necessarily the return value (need to apply other filters)
        should_run_check = (
            run_severity or
            explicit_run or
            implicit_run or
            (is_external and self.all_external)
        )

        if not should_run_check:
            logging.debug(f'Should run check {check_id}: False')
            return False

        # If a policy is not present in the list of filtered policies, it should not be run - implicitly or explicitly.
        # It can, however, be skipped.
        if not is_policy_filtered:
            logging.debug(f'not is_policy_filtered {check_id}: should_run_check = False')
            should_run_check = False

        skip_severity = severity and skip_check_threshold and severity.level <= skip_check_threshold.level
        explicit_skip = self.skip_checks and self.check_matches(check_id, bc_check_id, self.skip_checks)
        regex_match = self._match_regex_pattern(check_id, file_origin_paths, root_folder)
        should_skip_check = (
            skip_severity or
            explicit_skip or
            regex_match or
            (not bc_check_id and not self.include_all_checkov_policies and not is_external and not explicit_run) or
            (bc_check_id in self.suppressed_policies and bc_check_id not in self.bc_cloned_checks)
        )
        logging.debug(f'skip_severity = {skip_severity}, explicit_skip = {explicit_skip}, regex_match = {regex_match}, suppressed_policies: {self.suppressed_policies}')
        logging.debug(
            f'bc_check_id = {bc_check_id}, include_all_checkov_policies = {self.include_all_checkov_policies}, is_external = {is_external}, explicit_run: {explicit_run}')

        if should_skip_check:
            result = False
            logging.debug(f'should_skip_check {check_id}: {should_skip_check}')
        elif should_run_check:
            result = True
            logging.debug(f'should_run_check {check_id}: {result}')
        else:
            result = False
            logging.debug(f'default {check_id}: {result}')

        return result

    def _match_regex_pattern(self, check_id: str, file_origin_paths: List[str] | None, root_folder: str | None) -> bool:
        """
        Check if skip check_id for a certain file_types, according to given path pattern
        """
        if not file_origin_paths:
            return False
        regex_patterns = self.skip_checks_regex_patterns.get(check_id, [])
        # In case skip is generic, for example, CKV_AZURE_*.
        generic_check_id = f"{'_'.join(i for i in check_id.split('_')[:-1])}_*"
        generic_check_regex_patterns = self.skip_checks_regex_patterns.get(generic_check_id, [])
        regex_patterns.extend(generic_check_regex_patterns)
        if not regex_patterns:
            return False

        for pattern in regex_patterns:
            if not pattern:
                continue
            full_regex_pattern = fr"^{root_folder}/{pattern}" if root_folder else pattern
            try:
                if any(re.search(full_regex_pattern, path) for path in file_origin_paths):
                    return True
            except Exception as exc:
                logging.error(
                    "Invalid regex pattern has been supplied",
                    extra={"regex_pattern": pattern, "exc": str(exc)}
                )

        return False

    @staticmethod
    def check_matches(check_id: str,
                      bc_check_id: Optional[str],
                      pattern_list: List[str]) -> bool:
        return any(
            (fnmatch.fnmatch(check_id, pattern) or (bc_check_id and fnmatch.fnmatch(bc_check_id, pattern))) for pattern
            in pattern_list)

    def within_threshold(self, severity: Severity) -> bool:
        above_min = (not self.check_threshold) or self.check_threshold.level <= severity.level
        below_max = self.skip_check_threshold and self.skip_check_threshold.level >= severity.level
        return above_min and not below_max

    @staticmethod
    def secret_validation_status_matches(secret_validation_status: str, statuses_list: list[str]) -> bool:
        return secret_validation_status in statuses_list

    @staticmethod
    def notify_external_check(check_id: str) -> None:
        RunnerFilter.__EXTERNAL_CHECK_IDS.add(check_id)

    @staticmethod
    def is_external_check(check_id: str) -> bool:
        return check_id in RunnerFilter.__EXTERNAL_CHECK_IDS

    def is_policy_filtered(self, check_id: str) -> bool:
        if not self.filtered_policy_ids:
            return True
        return check_id in self.filtered_policy_ids

    def to_dict(self) -> Dict[str, Any]:
        result: Dict[str, Any] = {}
        for key, value in self.__dict__.items():
            result[key] = value
        return result

    @staticmethod
    def from_dict(obj: Dict[str, Any]) -> RunnerFilter:
        framework = obj.get('framework')
        checks = obj.get('checks')
        skip_checks = obj.get('skip_checks')
        include_all_checkov_policies = obj.get('include_all_checkov_policies')
        if include_all_checkov_policies is None:
            include_all_checkov_policies = True
        download_external_modules = obj.get('download_external_modules')
        if download_external_modules is None:
            download_external_modules = False
        external_modules_download_path = obj.get('external_modules_download_path')
        if external_modules_download_path is None:
            external_modules_download_path = DEFAULT_EXTERNAL_MODULES_DIR
        evaluate_variables = obj.get('evaluate_variables')
        if evaluate_variables is None:
            evaluate_variables = True
        runners = obj.get('runners')
        skip_framework = obj.get('skip_framework')
        excluded_paths = obj.get('excluded_paths')
        all_external = obj.get('all_external')
        if all_external is None:
            all_external = False
        var_files = obj.get('var_files')
        skip_cve_package = obj.get('skip_cve_package')
        use_enforcement_rules = obj.get('use_enforcement_rules')
        if use_enforcement_rules is None:
            use_enforcement_rules = False
        filtered_policy_ids = obj.get('filtered_policy_ids')
        show_progress_bar = obj.get('show_progress_bar')
        if show_progress_bar is None:
            show_progress_bar = True
        run_image_referencer = obj.get('run_image_referencer')
        if run_image_referencer is None:
            run_image_referencer = False
        enable_secret_scan_all_files = bool(obj.get('enable_secret_scan_all_files'))
        block_list_secret_scan = obj.get('block_list_secret_scan')
        runner_filter = RunnerFilter(framework, checks, skip_checks, include_all_checkov_policies,
                                     download_external_modules, external_modules_download_path, evaluate_variables,
                                     runners, skip_framework, excluded_paths, all_external, var_files,
                                     skip_cve_package, use_enforcement_rules, filtered_policy_ids, show_progress_bar,
                                     run_image_referencer, enable_secret_scan_all_files, block_list_secret_scan)
        return runner_filter

    def set_suppressed_policies(self, policy_level_suppressions: List[str]) -> None:
        logging.debug(f"Received the following policy-level suppressions, that will be skipped from running: {policy_level_suppressions}")
        self.suppressed_policies = policy_level_suppressions



from __future__ import annotations

import re
from collections.abc import Iterable

from checkov.common.bridgecrew.integration_features.features.policy_metadata_integration import (
    integration as metadata_integration,
)
from checkov.common.comment.enum import COMMENT_REGEX
from checkov.common.models.enums import CheckResult
from checkov.common.typing import _CheckResult, _SkippedCheck


def collect_suppressions_for_report(code_lines: list[tuple[int, str]]) -> dict[str, _CheckResult]:
    """Searches for suppressions in a config block to be used in a report"""

    suppressions = {}

    for _, line in code_lines:
        skip_search = re.search(COMMENT_REGEX, line)
        if skip_search:
            check_result: _CheckResult = {
                "result": CheckResult.SKIPPED,
                "suppress_comment": skip_search.group(3)[1:] if skip_search.group(3) else "No comment provided",
            }
            suppressions[skip_search.group(2)] = check_result

    return suppressions


def collect_suppressions_for_context(code_lines: Iterable[tuple[int, int | str]]) -> list[_SkippedCheck]:
    """Searches for suppressions in a config block to be used in a context"""

    skipped_checks = []
    bc_id_mapping = metadata_integration.bc_to_ckv_id_mapping
    for line_number, line_text in code_lines:
        skip_search = re.search(COMMENT_REGEX, str(line_text))
        if skip_search:
            skipped_check: _SkippedCheck = {
                "id": skip_search.group(2),
                "suppress_comment": skip_search.group(3)[1:] if skip_search.group(3) else "No comment provided",
                "line_number": line_number
            }
            # No matter which ID was used to skip, save the pair of IDs in the appropriate fields
            if bc_id_mapping and skipped_check["id"] in bc_id_mapping:
                skipped_check["bc_id"] = skipped_check["id"]
                skipped_check["id"] = bc_id_mapping[skipped_check["id"]]
            elif metadata_integration.check_metadata:
                skipped_check["bc_id"] = metadata_integration.get_bc_id(skipped_check["id"])

            skipped_checks.append(skipped_check)

    return skipped_checks



from __future__ import annotations

from typing import Any

from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class IAMUserNotUsedForAccess(BaseResourceCheck):
    def __init__(self) -> None:
        name = "Ensure access is controlled through SSO and not AWS IAM defined users"
        id = "CKV_AWS_273"
        supported_resources = ('aws_iam_user',)
        categories = (CheckCategories.IAM,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: dict[str, list[Any]]) -> CheckResult:
        return CheckResult.FAILED


check = IAMUserNotUsedForAccess()



from __future__ import annotations

from typing import Any

from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_negative_value_check import BaseResourceNegativeValueCheck


class ComputeInstanceGroupPublicIP(BaseResourceNegativeValueCheck):
    def __init__(self) -> None:
        name = "Ensure compute instance group does not have public IP."
        id = "CKV_YC_18"
        categories = (CheckCategories.NETWORKING,)
        supported_resources = ("yandex_compute_instance_group",)
        super().__init__(
            name=name,
            id=id,
            categories=categories,
            supported_resources=supported_resources,
        )

    def get_inspected_key(self) -> str:
        return "instance_template/[0]/network_interface/[0]/nat"

    def get_forbidden_values(self) -> list[Any]:
        return [True]


check = ComputeInstanceGroupPublicIP()



from checkov.terraform.checks.resource.ncp.AccessControlGroupInboundRule import AccessControlGroupInboundRule


class AccessControlGroupRuleInboundPort22(AccessControlGroupInboundRule):
    def __init__(self):
        super().__init__(check_id="CKV_NCP_4", port=22)


check = AccessControlGroupRuleInboundPort22()



from __future__ import annotations
from typing import Any

from checkov.common.images.workflow.image_referencer_manager import WorkflowImageReferencerManager
from checkov.github_actions.image_referencer.provider import GithubActionProvider


class GithubActionsImageReferencerManager(WorkflowImageReferencerManager):
    __slots__ = ("workflow_config", "file_path", "workflow_line_numbers", "provider")

    def __init__(self, workflow_config: dict[str, Any], file_path: str, workflow_line_numbers: list[tuple[int, str]]):
        provider = GithubActionProvider(workflow_config=workflow_config, file_path=file_path,
                                        workflow_line_numbers=workflow_line_numbers)
        super().__init__(workflow_config, file_path, provider)
        self.workflow_line_numbers = workflow_line_numbers



from __future__ import annotations

import datetime
import logging
import json
import subprocess  # nosec

from checkov.common.resource_code_logger_filter import add_resource_code_filter_to_logger
from checkov.common.util.http_utils import request_wrapper
from checkov.common.bridgecrew.platform_integration import BcPlatformIntegration
from typing import Any

logger = logging.getLogger(__name__)
add_resource_code_filter_to_logger(logger)


def report_contributor_metrics(repository: str, source: str,
                               bc_integration: BcPlatformIntegration) -> None:  # ignore: type
    logging.debug(f"Attempting to get log history for repository {repository} under source {source}")
    request_body = parse_gitlog(repository, source)
    number_of_attempts = 1
    contributors_report_api_url = f"{bc_integration.api_url}/api/v2/contributors/report"
    if request_body:
        while number_of_attempts <= 4:
            response = request_wrapper(
                "POST", contributors_report_api_url,
                headers=bc_integration.get_default_headers("POST"), data=json.dumps(request_body)
            )
            if response.status_code < 300:
                logging.debug(
                    f"Successfully uploaded contributor metrics with status: {response.status_code}. number of attempts: {number_of_attempts}")
                break
            else:
                contributors_report_api_url = f"{bc_integration.api_url}/api/v1/contributors/report"
                failed_attempt = {
                    'message': f"Failed to upload contributor metrics with: {response.status_code} - {response.reason}. number of attempts: {number_of_attempts}",
                    'timestamp': str(datetime.datetime.now())}
                request_body['failedAttempts'].append(failed_attempt)
                logging.info(f"Failed to upload contributor metrics with: {response.status_code} - {response.reason}")
                number_of_attempts += 1


def parse_gitlog(repository: str, source: str) -> dict[str, Any] | None:
    process = subprocess.Popen(['git', 'shortlog', '-ne', '--all', '--since', '"90 days ago"', '--pretty=commit-%ct', '--reverse'], stdout=subprocess.PIPE, stderr=subprocess.PIPE)  # nosec
    out, err = process.communicate()
    if err:
        logger.info(f"Failed to collect contributor metrics due to: {err}")  # type: ignore
        return {"repository": repository, "source": source,
                "contributors": [],
                "failedAttempts": [{
                    'message': f"Failed to collect contributor metrics due to: {err}",  # type: ignore
                    'timestamp': str(datetime.datetime.now())}]
                }
    # split per contributor
    list_of_contributors = out.decode('utf-8').split('\n\n')
    return {"repository": repository, "source": source,
            "contributors": list(map(lambda contributor: process_contributor(contributor),
                                     list(filter(lambda x: x, list_of_contributors))
                                     )),
            "failedAttempts": []
            }


def process_contributor(contributor: str) -> str:
    splittedList = contributor.split('\n')
    user = splittedList[0]
    commit = splittedList[1]
    return user[:user.find('(')] + commit[commit.find('-') + 1:]



from typing import Optional, Any, Dict
from collections.abc import Sized
from checkov.common.graph.checks_infra.enums import Operators
from checkov.common.checks_infra.solvers.attribute_solvers.base_attribute_solver import BaseAttributeSolver


class IsEmptyAttributeSolver(BaseAttributeSolver):
    operator = Operators.IS_EMPTY  # noqa: CCE003  # a static attribute

    def _get_operation(self, vertex: Dict[str, Any], attribute: Optional[str]) -> bool:
        attr = vertex.get(attribute)  # type:ignore[arg-type]  # due to attribute can be None

        if isinstance(attr, (list, Sized)):
            return len(attr) == 0

        return False



from __future__ import annotations

import logging
from collections.abc import Iterable
from typing import Any, Callable, TYPE_CHECKING  # noqa: F401  # Callable is used in the TypeAlias

from checkov.common.checks.base_check_registry import BaseCheckRegistry
from checkov.common.bridgecrew.check_type import CheckType
from checkov.common.resource_code_logger_filter import add_resource_code_filter_to_logger
from checkov.common.util.file_utils import read_file_with_any_encoding
from checkov.yaml_doc.runner import Runner as YamlRunner
from checkov.json_doc.runner import Runner as JsonRunner
from pathlib import Path

if TYPE_CHECKING:
    from typing_extensions import TypeAlias

_ParseFormatJsonCallable: TypeAlias = "Callable[[JsonRunner, str, str | None], tuple[dict[str, Any] | list[dict[str, Any]] | None, list[tuple[int, str]] | None] | None]"
_ParseFormatYamlCallable: TypeAlias = "Callable[[YamlRunner, str, str | None], tuple[dict[str, Any] | list[dict[str, Any]] | None, list[tuple[int, str]] | None] | None]"

logger = logging.getLogger(__name__)
add_resource_code_filter_to_logger(logger)


class Runner(YamlRunner, JsonRunner):
    check_type = CheckType.OPENAPI  # noqa: CCE003  # a static attribute

    def __init__(self) -> None:
        super().__init__()
        self.file_extensions = [".json", ".yml", ".yaml"]

    def import_registry(self) -> BaseCheckRegistry:
        from checkov.openapi.checks.registry import openapi_registry

        return openapi_registry

    def _parse_file(
        self, f: str, file_content: str | None = None
    ) -> tuple[dict[str, Any] | list[dict[str, Any]], list[tuple[int, str]]] | None:
        if f.endswith(".json"):
            return self.parse_format(f, JsonRunner._parse_file)
        elif f.endswith(".yml") or f.endswith(".yaml"):
            return self.parse_format(f, YamlRunner._parse_file)
        return None

    def parse_format(
        self,
        f: str,
        func: _ParseFormatJsonCallable | _ParseFormatYamlCallable,
    ) -> tuple[dict[str, Any] | list[dict[str, Any]], list[tuple[int, str]]] | None:
        try:
            content = self.load_file(f)
            valid_openapi_file = self.pre_validate_file(content)
            if not valid_openapi_file:
                return None

            parsed_file = func(self, f, content)

            if isinstance(parsed_file, tuple) and self.is_valid(parsed_file[0]):
                return parsed_file  # type:ignore[return-value]  # is_valid checks for being not empty
        except ValueError:
            logger.debug(f"Could not parse {f}, skipping file", exc_info=True)
        return None

    def get_start_end_lines(
        self, end: int, result_config: dict[str, Any] | list[dict[str, Any]], start: int
    ) -> tuple[int, int]:
        start_end_line: tuple[int, int]
        if hasattr(result_config, "start_mark"):
            start_end_line = JsonRunner.get_start_end_lines(self, end, result_config, start)  # type:ignore[arg-type]
            return start_end_line
        elif "__startline__" in result_config or isinstance(result_config, list):
            start_end_line = YamlRunner.get_start_end_lines(self, end, result_config, start)
            return start_end_line

        raise Exception("Unexpected dictionary format.")

    def require_external_checks(self) -> bool:
        return False

    def is_valid(self, conf: dict[str, Any] | list[dict[str, Any]] | None) -> bool:
        """validate openAPI configuration."""
        # 'swagger' is a required element on v2.0, and 'openapi' is required on v3.
        # 'info' object is required in v2.0 and v3:
        # https://swagger.io/specification/v2/#schema
        # https://swagger.io/specification/#schema
        try:
            return bool(
                conf
                and isinstance(conf, dict)
                and ("swagger" in conf or "openapi" in conf)
                and isinstance(conf["info"], dict)
            )
        except Exception:
            return False

    def get_resource(self, file_path: str, key: str, supported_entities: Iterable[str],
                     start_line: int = -1, end_line: int = -1, graph_resource: bool = False) -> str:
        return ",".join(supported_entities)

    def load_file(self, filename: str | Path) -> str:
        content = read_file_with_any_encoding(file_path=filename)
        return content

    def pre_validate_file(self, file_content: str) -> bool:
        openapi_keywords = ["swagger", "openapi"]
        match = any(keyword in file_content for keyword in openapi_keywords)
        if match:
            return True
        return False



from __future__ import annotations

from abc import ABC, abstractmethod
from typing import TYPE_CHECKING

from checkov.common.bridgecrew.integration_features.integration_feature_registry import integration_feature_registry

if TYPE_CHECKING:
    from argparse import Namespace
    from checkov.common.bridgecrew.platform_integration import BcPlatformIntegration
    from checkov.common.output.report import Report
    from checkov.common.typing import _BaseRunner


class BaseIntegrationFeature(ABC):
    def __init__(self, bc_integration: BcPlatformIntegration, order: int) -> None:
        self.bc_integration = bc_integration
        self.order = order
        integration_feature_registry.register(self)
        self.integration_feature_failures = False
        self.config: Namespace | None = None  # is set during pre_scan()

    @abstractmethod
    def is_valid(self) -> bool:
        raise NotImplementedError()

    @abstractmethod
    def pre_scan(self) -> None:
        """Runs before any runners"""
        pass

    @abstractmethod
    def pre_runner(self, runner: _BaseRunner) -> None:
        """Runs before each runner"""
        pass

    @abstractmethod
    def post_runner(self, scan_reports: Report) -> None:
        """Runs after each runner completes"""
        pass

    @abstractmethod
    def post_scan(self, merged_reports: list[Report]) -> Report | None:
        """Runs after all runners complete"""
        pass



from __future__ import annotations

from abc import ABC, abstractmethod
from typing import TYPE_CHECKING

from checkov.common.checks.base_check import BaseCheck
from checkov.common.util.tqdm_utils import ProgressBar

from checkov.common.output.report import Report
from checkov.policies_3d.checks_infra.base_check import Base3dPolicyCheck
from checkov.runner_filter import RunnerFilter

if TYPE_CHECKING:
    from checkov.common.graph.graph_manager import GraphManager  # noqa


class BasePostRunner(ABC):
    check_type = ''  # noqa: CCE003  # a static attribute

    def __init__(self) -> None:
        self.pbar = ProgressBar(self.check_type)

    @abstractmethod
    def run(
            self,
            checks: list[BaseCheck | Base3dPolicyCheck],
            scan_reports: list[Report],
            runner_filter: RunnerFilter | None = None
    ) -> Report:
        raise NotImplementedError()



from __future__ import annotations

import itertools
import logging
from collections import defaultdict
from dataclasses import dataclass
from typing import List, Union, Dict, Any

from prettytable import PrettyTable, SINGLE_BORDER

from checkov.common.bridgecrew.severities import BcSeverities
from checkov.common.models.enums import CheckResult
from checkov.common.output.common import compare_table_items_severity
from checkov.common.output.record import Record, SCA_PACKAGE_SCAN_CHECK_NAME, SCA_LICENSE_CHECK_NAME
from checkov.common.packaging import version as packaging_version
from checkov.common.sca.commons import UNFIXABLE_VERSION, get_package_alias
from checkov.common.typing import _LicenseStatus


@dataclass
class CveCount:
    total: int = 0
    critical: int = 0
    high: int = 0
    medium: int = 0
    low: int = 0
    skipped: int = 0
    has_fix: int = 0
    to_fix: int = 0
    fixable: bool = True

    def output_row(self) -> List[str]:
        return [
            f"Total CVEs: {self.total}",
            f"critical: {self.critical}",
            f"high: {self.high}",
            f"medium: {self.medium}",
            f"low: {self.low}",
            f"skipped: {self.skipped}",
        ]


def calculate_lowest_compliant_version(
    fix_versions_lists: List[List[Union[packaging_version.Version, packaging_version.LegacyVersion]]]
) -> str:
    """A best effort approach to find the lowest compliant version"""

    package_min_versions = set()
    package_versions = set()

    for fix_versions in fix_versions_lists:
        if fix_versions:
            package_min_versions.add(min(fix_versions))
            package_versions.update(fix_versions)
    if package_min_versions:
        package_min_version = min(package_min_versions)
        package_max_version = max(package_min_versions)

        if isinstance(package_min_version, packaging_version.LegacyVersion) or isinstance(
            package_max_version, packaging_version.LegacyVersion
        ):
            return str(package_max_version)
        elif package_min_version.major == package_max_version.major:
            return str(package_max_version)
        else:
            lowest_version = max(
                version
                for version in package_versions
                if isinstance(version, packaging_version.Version) and version.major == package_max_version.major
            )
            return str(lowest_version)

    return UNFIXABLE_VERSION


def create_cli_output(fixable: bool = True, *cve_records: list[Record]) -> str:
    cli_outputs = []
    group_by_file_path_package_map: dict[str, dict[str, list[Record]]] = defaultdict(dict)

    for record in itertools.chain(*cve_records):
        if not record.vulnerability_details:
            #  this shouldn't happen
            logging.error(f"'vulnerability_details' is not set for {record.check_id}")
            continue

        group_by_file_path_package_map[record.file_path].setdefault(
            record.vulnerability_details["package_name"], []
        ).append(record)

    for file_path, packages in group_by_file_path_package_map.items():
        cve_count = CveCount(fixable=fixable)
        package_cves_details_map: dict[str, dict[str, Any]] = defaultdict(dict)
        package_licenses_details_map = defaultdict(list)
        should_print_licenses_table = False
        for package_name, records in packages.items():
            package_version = None
            fix_versions_lists = []

            for record in records:
                if not record.vulnerability_details:
                    #  this shouldn't happen
                    logging.error(f"'vulnerability_details' is not set for {record.check_id}")
                    continue

                if record.check_name == SCA_PACKAGE_SCAN_CHECK_NAME:
                    cve_count.total += 1

                    if record.check_result["result"] == CheckResult.SKIPPED:
                        cve_count.skipped += 1
                        continue
                    else:
                        cve_count.to_fix += 1

                    # best way to dynamically access a class instance attribute.
                    # (we can't just do cve_count.severity_str to access the correct severity)
                    severity_str = record.severity.name.lower() if record.severity else BcSeverities.NONE.lower()
                    setattr(cve_count, severity_str, getattr(cve_count, severity_str) + 1)

                    if record.vulnerability_details["lowest_fixed_version"] != UNFIXABLE_VERSION:
                        cve_count.has_fix += 1

                    fix_versions_lists.append(record.vulnerability_details["fixed_versions"])
                    if package_version is None:
                        package_version = record.vulnerability_details["package_version"]

                    package_cves_details_map[package_name].setdefault("cves", []).append(
                        {
                            "id": record.vulnerability_details["id"],
                            "severity": severity_str,
                            "fixed_version": record.vulnerability_details["lowest_fixed_version"],
                        }
                    )
                elif record.check_name == SCA_LICENSE_CHECK_NAME:
                    if record.check_result["result"] == CheckResult.SKIPPED:
                        continue
                    should_print_licenses_table = True
                    package_licenses_details_map[get_package_alias(package_name, record.vulnerability_details["package_version"])].append(
                        _LicenseStatus(package_name=package_name,
                                       package_version=record.vulnerability_details["package_version"],
                                       policy=record.vulnerability_details["policy"],
                                       license=record.vulnerability_details["license"],
                                       status=record.vulnerability_details["status"])
                    )

            if package_name in package_cves_details_map:
                package_cves_details_map[package_name]["cves"].sort(key=compare_table_items_severity, reverse=True)
                package_cves_details_map[package_name]["current_version"] = package_version
                package_cves_details_map[package_name]["compliant_version"] = calculate_lowest_compliant_version(
                    fix_versions_lists
                )

        if cve_count.total > 0:
            cli_outputs.append(
                create_cli_cves_table(
                    file_path=file_path,
                    cve_count=cve_count,
                    package_details_map=package_cves_details_map,
                )
            )
        if should_print_licenses_table:
            cli_outputs.append(
                create_cli_license_violations_table(
                    file_path=file_path,
                    package_licenses_details_map=package_licenses_details_map
                )
            )

    return "\n".join(cli_outputs)


def create_cli_license_violations_table(file_path: str, package_licenses_details_map: Dict[str, List[_LicenseStatus]]) -> str:
    package_table_lines: List[str] = []
    columns = 5
    table_width = 126
    column_width = int(table_width / columns)
    package_table = PrettyTable(min_table_width=table_width, max_table_width=table_width)
    package_table.set_style(SINGLE_BORDER)
    package_table.field_names = [
        "Package name",
        "Package version",
        "Policy ID",
        "License",
        "Status",
    ]
    for package_idx, (_, license_statuses) in enumerate(package_licenses_details_map.items()):
        if package_idx > 0:
            del package_table_lines[-1]
            package_table.header = False
            package_table.clear_rows()

        for idx, license_status in enumerate(license_statuses):
            col_package_name = ""
            col_package_version = ""
            if idx == 0:
                col_package_name = license_status["package_name"]
                col_package_version = license_status["package_version"]

            package_table.add_row(
                [
                    col_package_name,
                    col_package_version,
                    license_status["policy"],
                    license_status["license"],
                    license_status["status"],
                ]
            )

        package_table.align = "l"
        package_table.min_width = column_width
        package_table.max_width = column_width

        for idx, line in enumerate(package_table.get_string().splitlines(keepends=True)):
            if idx == 0 and package_idx != 0:
                # hack to make multiple tables look like one
                line = line.replace(package_table.top_left_junction_char, package_table.left_junction_char).replace(
                    package_table.top_right_junction_char, package_table.right_junction_char
                )
            if package_idx > 0:
                # hack to make multiple package tables look like one
                line = line.replace(package_table.top_junction_char, package_table.junction_char)

            # hack for making the table's width as same as the cves-table's
            package_table_lines.append(f"\t{line[:-2]}{line[-3]}{line[-2:]}")

    return (
        f"\t{file_path} - Licenses Statuses:\n"
        f"{''.join(package_table_lines)}\n"
    )


def create_cli_cves_table(file_path: str, cve_count: CveCount, package_details_map: Dict[str, Dict[str, Any]]) -> str:
    columns = 6
    table_width = 126
    column_width = int(126 / columns)

    cve_table_lines = create_cve_summary_table_part(
        table_width=table_width, column_width=column_width, cve_count=cve_count
    )

    vulnerable_packages = True if package_details_map else False
    fixable_table_lines = create_fixable_cve_summary_table_part(
        table_width=table_width, cve_count=cve_count, vulnerable_packages=vulnerable_packages
    )

    package_table_lines = create_package_overview_table_part(
        table_width=table_width, column_width=column_width, package_details_map=package_details_map
    )

    return (
        f"\t{file_path} - CVEs Summary:\n"
        f"{''.join(cve_table_lines)}\n"
        f"{''.join(fixable_table_lines)}"
        f"{''.join(package_table_lines)}\n"
    )


def create_cve_summary_table_part(table_width: int, column_width: int, cve_count: CveCount) -> List[str]:
    cve_table = PrettyTable(
        header=False,
        padding_width=1,
        min_table_width=table_width,
        max_table_width=table_width,
    )
    cve_table.set_style(SINGLE_BORDER)
    cve_table.add_row(cve_count.output_row())
    cve_table.align = "l"
    cve_table.min_width = column_width
    cve_table.max_width = column_width

    cve_table_lines = [f"\t{line}" for line in cve_table.get_string().splitlines(keepends=True)]
    # hack to make multiple tables look like one
    cve_table_bottom_line = (
        cve_table_lines[-1]
        .replace(cve_table.bottom_left_junction_char, cve_table.left_junction_char)
        .replace(cve_table.bottom_right_junction_char, cve_table.right_junction_char)
    )
    cve_table_lines[-1] = cve_table_bottom_line

    return cve_table_lines


def create_fixable_cve_summary_table_part(
        table_width: int, cve_count: CveCount, vulnerable_packages: bool
) -> List[str]:
    fixable_table = PrettyTable(
        header=False, min_table_width=table_width + 1, max_table_width=table_width + 1
    )
    fixable_table.set_style(SINGLE_BORDER)
    if cve_count.fixable:
        fixable_table.add_row(
            [f"To fix {cve_count.has_fix}/{cve_count.to_fix} CVEs, go to https://www.bridgecrew.cloud/"])
        fixable_table.align = "l"

    # hack to make multiple tables look like one
    fixable_table_lines = [f"\t{line}" for line in fixable_table.get_string().splitlines(keepends=True)]
    del fixable_table_lines[0]
    # only remove the last line, if there are vulnerable packages
    if vulnerable_packages:
        del fixable_table_lines[-1]

    return fixable_table_lines


def create_package_overview_table_part(
        table_width: int, column_width: int, package_details_map: Dict[str, Dict[str, Any]]
) -> List[str]:
    package_table_lines: List[str] = []
    package_table = PrettyTable(min_table_width=table_width, max_table_width=table_width)
    package_table.set_style(SINGLE_BORDER)
    package_table.field_names = [
        "Package",
        "CVE ID",
        "Severity",
        "Current version",
        "Fixed version",
        "Compliant version",
    ]
    for package_idx, (package_name, details) in enumerate(package_details_map.items()):
        if package_idx > 0:
            del package_table_lines[-1]
            package_table.header = False
            package_table.clear_rows()

        for cve_idx, cve in enumerate(details["cves"]):
            col_package = ""
            col_current_version = ""
            col_compliant_version = ""
            if cve_idx == 0:
                col_package = package_name
                col_current_version = details["current_version"]
                col_compliant_version = details["compliant_version"]

            package_table.add_row(
                [
                    col_package,
                    cve["id"],
                    cve["severity"],
                    col_current_version,
                    cve["fixed_version"],
                    col_compliant_version,
                ]
            )

        package_table.align = "l"
        package_table.min_width = column_width
        package_table.max_width = column_width

        for idx, line in enumerate(package_table.get_string().splitlines(keepends=True)):
            if idx == 0:
                # hack to make multiple tables look like one
                line = line.replace(package_table.top_left_junction_char, package_table.left_junction_char).replace(
                    package_table.top_right_junction_char, package_table.right_junction_char
                )
            if package_idx > 0:
                # hack to make multiple package tables look like one
                line = line.replace(package_table.top_junction_char, package_table.junction_char)

            package_table_lines.append(f"\t{line}")

    return package_table_lines



from typing import Any

from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck
from checkov.common.models.consts import ANY_VALUE


class AutoScalingLaunchTemplate(BaseResourceValueCheck):
    def __init__(self) -> None:
        """
        NIST.800-53.r5 CA-9(1), NIST.800-53.r5 CM-2, NIST.800-53.r5 CM-2(2)
        EC2 Auto Scaling groups should use EC2 launch templates
        """
        name = "Ensure EC2 Auto Scaling groups use EC2 launch templates"
        id = "CKV_AWS_315"
        supported_resources = ("aws_autoscaling_group",)
        categories = (CheckCategories.GENERAL_SECURITY,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        return "launch_template"

    def get_expected_value(self) -> Any:
        return ANY_VALUE


check = AutoScalingLaunchTemplate()



from __future__ import annotations

from typing import Any

from checkov.bicep.checks.resource.base_resource_check import BaseResourceCheck
from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.common.util.type_forcers import force_int


class StorageAccountsTransportEncryption(BaseResourceCheck):
    def __init__(self) -> None:
        name = "Ensure that 'supportsHttpsTrafficOnly' is set to 'true'"
        id = "CKV_AZURE_3"
        supported_resources = ("Microsoft.Storage/storageAccounts",)
        categories = (CheckCategories.ENCRYPTION,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: dict[str, Any]) -> CheckResult:
        self.evaluated_keys = ["properties/supportsHttpsTrafficOnly"]
        properties = conf.get("properties")
        if properties:
            if not isinstance(properties, dict):
                return CheckResult.UNKNOWN

            https_only = properties.get("supportsHttpsTrafficOnly")
            if https_only is True:
                return CheckResult.PASSED
            elif https_only is False:
                return CheckResult.FAILED

        year = force_int(self.api_version[:4])
        if year is None:
            return CheckResult.UNKNOWN
        elif year < 2019:
            return CheckResult.FAILED

        return CheckResult.PASSED


check = StorageAccountsTransportEncryption()



from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class GoogleCloudSqlServerNoPublicIP(BaseResourceCheck):
    def __init__(self):
        name = "Ensure Cloud SQL database does not have public IP"
        check_id = "CKV_GCP_60"
        supported_resources = ['google_sql_database_instance']
        categories = [CheckCategories.NETWORKING]
        super().__init__(name=name, id=check_id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
        """
            Looks for google_sql_database_instance which have no public IPs on SQL DBs::
            :param
            conf: google_sql_database_instance
            configuration
            :return: < CheckResult >
        """

        if 'settings' in conf.keys() and 'ip_configuration' in conf['settings'][0] and \
                'ipv4_enabled' in conf['settings'][0]['ip_configuration'][0]:
            ipconfiguration = conf['settings'][0]['ip_configuration'][0]
            ipv4_enabled = ipconfiguration['ipv4_enabled']
            ipv4_enabled = ipv4_enabled[0] if isinstance(ipv4_enabled, list) else ipv4_enabled
            if ipv4_enabled:
                self.evaluated_keys = ['database_version/[0]/SQLSERVER',
                                       'settings/[0]/ip_configuration/[0]/ipv4_enabled']
                return CheckResult.FAILED
        return CheckResult.PASSED


check = GoogleCloudSqlServerNoPublicIP()



from __future__ import annotations

from typing import TYPE_CHECKING

from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.dockerfile.base_dockerfile_check import BaseDockerfileCheck

if TYPE_CHECKING:
    from dockerfile_parse.parser import _Instruction


class AliasIsUnique(BaseDockerfileCheck):
    def __init__(self) -> None:
        """
        Ensure From Alias are unique for multistage builds.
        """
        name = "Ensure From Alias are unique for multistage builds."
        id = "CKV_DOCKER_11"
        supported_instructions = ("FROM",)
        categories = (CheckCategories.CONVENTION,)
        super().__init__(name=name, id=id, categories=categories, supported_instructions=supported_instructions)

    def scan_resource_conf(self, conf: list[_Instruction]) -> tuple[CheckResult, list[_Instruction] | None]:
        alias = []
        for instruction in conf:
            if " as " in instruction["value"]:
                alias.append(instruction["value"].rsplit(maxsplit=1)[-1])

        if len(alias) == len(set(alias)):
            return CheckResult.PASSED, None

        return CheckResult.FAILED, [conf[0]]


check = AliasIsUnique()



from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck
from checkov.common.models.enums import CheckCategories, CheckResult
from typing import Any


class ACMCertSetLoggingPreference(BaseResourceValueCheck):

    def __init__(self):
        """
        To guard against SSL/TLS certificates that are issued by mistake or by a compromised CA,
        some browsers require that public certificates issued for your domain be recorded in a certificate
        transparency log.
         The domain name is recorded. The private key is not.
         Certificates that are not logged typically generate an error in the browser
        """
        name = "Verify logging preference for ACM certificates"
        id = "CKV_AWS_234"
        supported_resources = ['aws_acm_certificate']
        categories = [CheckCategories.LOGGING]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources,
                         missing_block_result=CheckResult.PASSED)

    def get_inspected_key(self):
        return "options/[0]/certificate_transparency_logging_preference"

    def get_expected_value(self) -> Any:
        return "ENABLED"


check = ACMCertSetLoggingPreference()



from __future__ import annotations

from typing import Any, TYPE_CHECKING, cast

import jmespath

from checkov.bitbucket_pipelines.registry import registry
from checkov.common.images.image_referencer import ImageReferencer, Image
from checkov.common.bridgecrew.check_type import CheckType
from checkov.yaml_doc.runner import Runner as YamlRunner

if TYPE_CHECKING:
    from checkov.common.checks.base_check_registry import BaseCheckRegistry


class Runner(YamlRunner, ImageReferencer):
    check_type = CheckType.BITBUCKET_PIPELINES  # noqa: CCE003  # a static attribute

    def __init__(self) -> None:
        super().__init__()

    def require_external_checks(self) -> bool:
        return False

    def import_registry(self) -> BaseCheckRegistry:
        return registry

    def _parse_file(
        self, f: str, file_content: str | None = None
    ) -> tuple[dict[str, Any] | list[dict[str, Any]], list[tuple[int, str]]] | None:
        if self.is_workflow_file(f):
            return super()._parse_file(f)

        return None

    def is_workflow_file(self, file_path: str) -> bool:
        """
        :return: True if the file mentioned is named bitbucket-pipelines.yml. Otherwise: False
        """
        return file_path.endswith(("bitbucket-pipelines.yml", "bitbucket-pipelines.yaml"))

    def get_images(self, file_path: str) -> set[Image]:
        """
        Get container images mentioned in a file
        :param file_path: File to be inspected

        File sample that will return 4 Image objects:
        #image: node:10.15.0
        #
        #pipelines:
        #  default:
        #    - step:
        #        name: Build and test
        #        image: node:10.0.0
        #        script:
        #          - npm install
        #          - npm test
        #          - npm run build
        #        artifacts:
        #          - dist/**
        #    - step:
        #        name: Deploy
        #        image: python:3.7.2
        #        trigger: manual
        #        script:
        #          - python deploy.py
        #  custom:
        #    sonar:
        #      - step:
        #          image: python:3.8.2
        #          script:
        #            - echo "Manual triggers for Sonar are awesome!"
        #    deployment-to-prod:
        #      - step:
        #          script:
        #            - echo "Manual triggers for deployments are awesome!"
        #  branches:
        #    staging:
        #      - step:
        #          script:
        #            - echo "Auto pipelines are cool too."
        :return: List of container image objects mentioned in the file.

        """

        images: set[Image] = set()
        parsed_file = self._parse_file(file_path)

        if not parsed_file:
            return images

        workflow, workflow_line_numbers = parsed_file

        if not isinstance(workflow, dict):
            # make type checking happy
            return images

        self.add_default_and_pipelines_images(workflow, images, file_path)
        self.add_root_image(file_path, images, workflow_line_numbers, workflow)

        return images

    def add_default_and_pipelines_images(self, workflow: dict[str, Any], images: set[Image], file_path: str) -> None:
        """

        :param workflow: parsed workflow file
        :param images: set of images to be updated
        :param file_path: path of analyzed workflow
        """
        keywords = [
            "pipelines.default[].step.{image: image, __startline__: __startline__, __endline__:__endline__}",
            "pipelines.*.[*][][][].step.{image: image, __startline__: __startline__, __endline__:__endline__}",
        ]
        for keyword in keywords:
            results = cast("list[dict[str, Any]]", jmespath.search(keyword, workflow))
            for result in results:
                image_name = result.get("image", None)
                if image_name:
                    image_obj = Image(
                        file_path=file_path,
                        name=image_name,
                        start_line=result["__startline__"],
                        end_line=result["__endline__"],
                    )
                    images.add(image_obj)

    def add_root_image(
        self, file_path: str, images: set[Image], workflow_line_numbers: list[tuple[int, str]], workflow: dict[str, Any]
    ) -> None:
        root_image = workflow.get("image", "")

        if root_image:
            for line_number, line_txt in workflow_line_numbers:
                if "image" in line_txt and not line_txt.startswith(" "):
                    image_obj = Image(
                        file_path=file_path,
                        name=root_image,
                        start_line=line_number,
                        end_line=line_number,
                    )
                    images.add(image_obj)



from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck
from checkov.common.models.consts import ANY_VALUE


class KendraIndexSSEUsesCMK(BaseResourceValueCheck):
    def __init__(self):
        name = "Ensure Kendra index Server side encryption uses CMK"
        id = "CKV_AWS_262"
        supported_resources = ['aws_kendra_index']
        categories = [CheckCategories.ENCRYPTION]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self):
        return 'server_side_encryption_configuration/[0]/kms_key_id'

    def get_expected_value(self):
        return ANY_VALUE


check = KendraIndexSSEUsesCMK()



from __future__ import annotations

from typing import Any

from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.cloudformation.checks.resource.base_resource_check import BaseResourceCheck
from checkov.cloudformation.parser.cfn_keywords import ConditionFunctions, IntrinsicFunctions


class ALBListenerHTTPS(BaseResourceCheck):
    def __init__(self) -> None:
        name = "Ensure ALB protocol is HTTPS"
        id = "CKV_AWS_2"
        supported_resources = ("AWS::ElasticLoadBalancingV2::Listener",)
        categories = (CheckCategories.ENCRYPTION,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: dict[str, Any]) -> CheckResult:
        """
            validates ALB protocol is HTTPS
            https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-elasticloadbalancingv2-listener.html
        :param conf: aws_alb_listener configuration
        :return: <CheckResult>
        """
        properties = conf.get("Properties")
        if properties and isinstance(properties, dict):
            protocol = properties.get("Protocol")
            if protocol:
                if protocol in ("HTTPS", "TLS", "TCP", "UDP", "TCP_UDP"):
                    return CheckResult.PASSED
                else:
                    if isinstance(properties.get("DefaultActions", {}), dict):
                        default_actions = properties.get("DefaultActions", {}).keys()
                        if any(
                            action in ConditionFunctions.__dict__.values()
                            or action in IntrinsicFunctions.__dict__.values()
                            for action in default_actions
                        ):
                            return CheckResult.UNKNOWN
                    if (
                        "DefaultActions" in properties.keys()
                        and properties["DefaultActions"][0].get("Type") == "redirect"
                        and properties["DefaultActions"][0].get("RedirectConfig", {}).get("Protocol") == "HTTPS"
                    ):
                        return CheckResult.PASSED
        return CheckResult.FAILED


check = ALBListenerHTTPS()



from __future__ import annotations

from abc import abstractmethod
from collections.abc import Iterable
from typing import TYPE_CHECKING, Any, Callable

from checkov.arm.registry import arm_parameter_registry
from checkov.common.checks.base_check import BaseCheck
from checkov.common.multi_signature import multi_signature

if TYPE_CHECKING:
    from checkov.common.models.enums import CheckCategories, CheckResult


class BaseParameterCheck(BaseCheck):
    def __init__(
        self,
        name: str,
        id: str,
        categories: Iterable[CheckCategories],
        supported_resources: Iterable[str],
        guideline: str | None = None
    ) -> None:
        super().__init__(name=name, id=id, categories=categories, supported_entities=supported_resources,
                         block_type="parameter", guideline=guideline)
        self.supported_resources = supported_resources
        arm_parameter_registry.register(self)

    def scan_entity_conf(self, conf: dict[str, Any], entity_type: str) -> CheckResult:  # type:ignore[override]  # it's ok
        return self.scan_resource_conf(conf, entity_type)  # type:ignore[no-any-return]  # issue with multi_signature annotation

    @multi_signature()
    @abstractmethod
    def scan_resource_conf(self, conf: dict[str, Any], entity_type: str) -> CheckResult:
        raise NotImplementedError()

    @classmethod
    @scan_resource_conf.add_signature(args=["self", "conf"])
    def _scan_resource_conf_self_conf(cls, wrapped: Callable[..., CheckResult]) -> Callable[..., CheckResult]:
        def wrapper(self: BaseCheck, conf: dict[str, Any], entity_type: str | None = None) -> CheckResult:
            # keep default argument for entity_type so old code, that doesn't set it, will work.
            return wrapped(self, conf)

        return wrapper



from __future__ import annotations

from typing import List, Any

from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class LambdaServicePermission(BaseResourceCheck):
    def __init__(self) -> None:
        description = "Ensure that AWS Lambda function permissions delegated to AWS services are limited by SourceArn or SourceAccount"
        id = "CKV_AWS_364"
        supported_resources = ('aws_lambda_permission',)
        categories = (CheckCategories.IAM,)
        super().__init__(name=description, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: dict[str, list[Any]]) -> CheckResult:
        # Replace this with the custom logic for your check
        principal = conf.get("principal")
        if principal and isinstance(principal, list) and isinstance(principal[0], str):
            principal_parts = principal[0].split('.')
            try:
                if principal_parts[1] == 'amazonaws' and principal_parts[2] == 'com':  # This confirms that the principal is set as a service principal.
                    if 'source_arn' in conf or 'source_account' in conf:  # If either of these are set, we're good and the check should pass.
                        self.evaluated_keys = self.get_evaluated_keys()
                        return CheckResult.PASSED
                    else:
                        self.evaluated_keys = self.get_evaluated_keys()
                        return CheckResult.FAILED
            except IndexError:
                return CheckResult.UNKNOWN
        return CheckResult.UNKNOWN

    def get_evaluated_keys(self) -> List[str]:
        return ["principal", "source_arn", "source_account"]


check = LambdaServicePermission()



from __future__ import annotations

from typing import Any

from checkov.common.bridgecrew.check_type import CheckType
from checkov.common.checks.base_check_registry import BaseCheckRegistry


class Registry(BaseCheckRegistry):
    def __init__(self) -> None:
        super().__init__(report_type=CheckType.CLOUDFORMATION)

    def extract_entity_details(self, entity: dict[str, dict[str, Any]]) -> tuple[str, str, dict[str, Any]]:
        resource_name, resource = next(iter(entity.items()))
        resource_type = resource["Type"]
        return resource_type, resource_name, resource



from __future__ import annotations

from typing import Any

from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.arm.base_resource_check import BaseResourceCheck
from checkov.common.util.data_structures_utils import find_in_dict


class VMScaleSetsAutoOSImagePatchingEnabled(BaseResourceCheck):
    def __init__(self) -> None:
        name = "Ensure that automatic OS image patching is enabled for Virtual Machine Scale Sets"
        id = "CKV_AZURE_95"
        supported_resources = ("Microsoft.Compute/virtualMachineScaleSets",)
        categories = (CheckCategories.GENERAL_SECURITY,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: dict[str, Any]) -> CheckResult:
        properties = conf.get("properties")
        if properties and isinstance(properties, dict):
            if properties.get("orchestrationMode") == "Flexible":
                self.evaluated_keys = ["properties/orchestrationMode"]
                return CheckResult.FAILED

            self.evaluated_keys = ["properties/virtualMachineProfile/extensionProfile/extensions"]
            extensions = find_in_dict(
                input_dict=properties,
                key_path="virtualMachineProfile/extensionProfile/extensions",
            )
            if extensions:
                for extension in extensions:
                    extension_properties = extension.get("properties")
                    if extension_properties and isinstance(extension_properties, dict):
                        if extension_properties.get("enableAutomaticUpgrade") is True:
                            return CheckResult.PASSED

            return CheckResult.FAILED

        return CheckResult.UNKNOWN


check = VMScaleSetsAutoOSImagePatchingEnabled()



from __future__ import annotations

import typing

from checkov.terraform.graph_builder.foreach.module_handler import ForeachModuleHandler
from checkov.terraform.graph_builder.foreach.resource_handler import ForeachResourceHandler
from checkov.terraform.graph_builder.graph_components.block_types import BlockType

if typing.TYPE_CHECKING:
    from checkov.terraform.graph_builder.local_graph import TerraformLocalGraph


class ForeachBuilder:
    def __init__(self, local_graph: TerraformLocalGraph):
        self._resource_handler = ForeachResourceHandler(local_graph)
        self._module_handler = ForeachModuleHandler(local_graph)

    def handle(self, foreach_blocks: dict[str, list[int]]) -> None:
        if self._module_handler.local_graph.enable_modules_foreach_handling:
            if foreach_blocks.get(BlockType.MODULE):
                self._module_handler.handle(foreach_blocks[BlockType.MODULE])
                self._module_handler.local_graph._arrange_graph_data()
                self._module_handler.local_graph._build_edges()
        if self._module_handler.local_graph.enable_foreach_handling:
            self._resource_handler.handle(foreach_blocks.get(BlockType.RESOURCE, []))



from __future__ import annotations

import json
import logging
import os
import re
from pathlib import Path
from typing import List, Callable

from checkov.common.parallelizer.parallel_runner import parallel_runner
from checkov.common.util.file_utils import read_file_with_any_encoding
from checkov.common.util.type_forcers import convert_str_to_bool
from checkov.terraform.module_loading.registry import module_loader_registry

MODULE_NAME_PATTERN = re.compile(r'[^#]*\bmodule\s*"(?P<name>.*)"')
MODULE_SOURCE_PATTERN = re.compile(r'[^#]*\bsource\s*=\s*"(?P<link>.*)"')
MODULE_VERSION_PATTERN = re.compile(r'[^#]*\bversion\s*=\s*"(?P<operator>=|!=|>=|>|<=|<|~>\s*)?(?P<version>[\d.]+-?\w*)"')


class ModuleDownload:
    def __init__(self, source_dir: str) -> None:
        self.source_dir = source_dir
        self.address: str | None = None
        self.module_name: str | None = None
        self.module_link: str | None = None
        self.tf_managed = False
        self.version: str | None = None

    def __str__(self) -> str:
        return f"{self.source_dir} -> {self.module_link} ({self.version})"


def find_modules(path: str) -> List[ModuleDownload]:
    modules_found: list[ModuleDownload] = []

    for root, _, full_file_names in os.walk(path):
        for file_name in full_file_names:
            if not file_name.endswith('.tf'):
                continue
            if root.startswith(os.path.join(path, ".terraform", "modules")):
                # don't scan the modules folder used by Terraform
                continue

            try:
                content = read_file_with_any_encoding(file_path=os.path.join(path, root, file_name))
                if "module " not in content:
                    # if there is no "module " ref in the whole file, then no need to search line by line
                    continue

                curr_md = None
                for line in content.splitlines():
                    if not curr_md:
                        if line.startswith('module'):
                            curr_md = ModuleDownload(os.path.dirname(os.path.join(root, file_name)))

                            # also extract the name for easier mapping against the TF modules.json file
                            match = re.match(MODULE_NAME_PATTERN, line)
                            if match:
                                curr_md.module_name = match.group("name")

                            continue
                    else:
                        if line.startswith('}'):
                            if curr_md.module_link is None:
                                logging.warning(f'A module at {curr_md.source_dir} had no source, skipping')
                            else:
                                curr_md.address = f"{curr_md.module_link}:{curr_md.version}"
                                modules_found.append(curr_md)
                            curr_md = None
                            continue

                        if "source" in line:
                            match = re.match(MODULE_SOURCE_PATTERN, line)
                            if match:
                                curr_md.module_link = match.group('link')
                                continue

                        if "version" in line:
                            match = re.match(MODULE_VERSION_PATTERN, line)
                            if match:
                                curr_md.version = f"{match.group('operator')}{match.group('version')}" if match.group('operator') else match.group('version')
            except (UnicodeDecodeError, FileNotFoundError) as e:
                logging.warning(f"Skipping {os.path.join(path, root, file_name)} because of {e}")
                continue

    return modules_found


def should_download(path: str | None) -> bool:

    return path is not None and not (path.startswith('./') or path.startswith('../') or path.startswith('/'))


def load_tf_modules(
    path: str,
    should_download_module: Callable[[str | None], bool] = should_download,
    run_parallel: bool = False,
    modules_to_load: List[ModuleDownload] | None = None,
    stop_on_failure: bool = False
) -> None:
    module_loader_registry.root_dir = path
    if not modules_to_load:
        modules_to_load = find_modules(path)

    def _download_module(m: ModuleDownload) -> bool:
        if should_download_module(m.module_link):
            logging.info(f'Downloading module {m.address}')
            try:
                content = module_loader_registry.load(
                    current_dir=m.source_dir,
                    source=m.module_link,
                    source_version="latest" if not m.version else m.version,
                    module_address=m.address,
                    tf_managed=m.tf_managed,
                )
                if content is None or not content.loaded():
                    log_message = f'Failed to download module {m.address}'
                    if not module_loader_registry.download_external_modules:
                        log_message += ' (for external modules, the --download-external-modules flag is required)'
                    logging.warning(log_message)
                    return False
            except Exception as e:
                logging.warning(f"Unable to load module ({m.address}): {e}")
                return False
        return True

    # To avoid duplicate work, we need to get the distinct module sources
    distinct_modules = list({m.address: m for m in modules_to_load}.values())

    replaced_modules = replace_terraform_managed_modules(path=path, found_modules=distinct_modules)

    if run_parallel:
        list(parallel_runner.run_function(_download_module, replaced_modules))
    else:
        logging.info(f"Starting download of modules of length {len(replaced_modules)}")
        for m in replaced_modules:
            success = _download_module(m)
            if not success and stop_on_failure:
                logging.info(f"Stopping downloading of modules due to failed attempt on {m.address}")
                break


def replace_terraform_managed_modules(path: str, found_modules: list[ModuleDownload]) -> list[ModuleDownload]:
    """Replaces modules by Terraform managed ones to prevent addtional downloading

    It can't handle nested modules yet, ex.
    {
      "Key": "parent_module.child_module",
      "Source": "./child_module",
      "Dir": "parent_module/child_module"
    }
    """

    if not convert_str_to_bool(os.getenv("CHECKOV_EXPERIMENTAL_TERRAFORM_MANAGED_MODULES", False)):
        return found_modules

    # file used by Terraform internally to map modules to the downloaded path
    tf_modules_file = Path(path) / ".terraform/modules/modules.json"
    if not tf_modules_file.exists():
        return found_modules

    # create Key (module name) to module detail map for faster querying
    tf_modules = {
        module["Key"]: module
        for module in json.loads(tf_modules_file.read_bytes())["Modules"]
    }

    replaced_modules: list[ModuleDownload] = []
    for module in found_modules:
        if module.module_name in tf_modules:
            tf_module = tf_modules[module.module_name]

            module_new = ModuleDownload(source_dir=path)
            # if version is 'None' then set it to latest in the address, so it can be mapped properly later on
            module_new.address = f"{module.module_link}:latest" if module.version is None else module.address
            module_new.module_link = tf_module["Dir"]
            module_new.module_name = module.module_name
            module_new.tf_managed = True
            module_new.version = module.version

            replaced_modules.append(module_new)
        else:
            replaced_modules.append(module)

    return replaced_modules



from __future__ import annotations

from typing import Any
from checkov.common.models.consts import ANY_VALUE
from checkov.common.models.enums import CheckCategories
from checkov.arm.base_resource_value_check import BaseResourceValueCheck


class SecretContentType(BaseResourceValueCheck):
    def __init__(self):
        name = "Ensure that key vault secrets have \"content_type\" set"
        id = "CKV_AZURE_114"
        supported_resources = ['Microsoft.KeyVault/vaults/secrets']
        categories = [CheckCategories.GENERAL_SECURITY]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        return "properties/contentType"

    def get_expected_value(self) -> Any:
        return ANY_VALUE


check = SecretContentType()



from __future__ import annotations

from typing import Any
from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.arm.base_resource_check import BaseResourceCheck


class MySQLEncryptionEnabled(BaseResourceCheck):
    def __init__(self) -> None:
        name = "Ensure that MySQL server enables infrastructure encryption"
        id = "CKV_AZURE_96"
        supported_resources = ("Microsoft.DBforMySQL/flexibleServers",)
        categories = (CheckCategories.ENCRYPTION,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: dict[str, Any], entity_type: str) -> CheckResult:
        properties = conf.get("properties")
        if properties and isinstance(properties, dict):
            self.evaluated_keys = ["properties/dataencryption"]
            data_encryption = properties.get("dataencryption")
            if data_encryption and isinstance(data_encryption, dict):
                if data_encryption is None:
                    return CheckResult.FAILED
                return CheckResult.PASSED
            # unparsed
            elif data_encryption and isinstance(data_encryption, str):
                return CheckResult.UNKNOWN
            return CheckResult.FAILED
        return CheckResult.UNKNOWN


check = MySQLEncryptionEnabled()



from typing import Any

from checkov.common.models.consts import ANY_VALUE
from checkov.common.models.enums import CheckCategories
from checkov.arm.base_resource_value_check import BaseResourceValueCheck


class AzureFrontDoorEnablesWAF(BaseResourceValueCheck):
    def __init__(self) -> None:
        name = "Ensure that Azure Front Door enables WAF"
        id = "CKV_AZURE_121"
        supported_resources = ("Microsoft.Network/frontDoors",)
        categories = [CheckCategories.NETWORKING]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        return "properties/frontendEndpoints/[0]/properties/webApplicationFirewallPolicyLink/id"

    def get_expected_value(self) -> Any:
        return ANY_VALUE


check = AzureFrontDoorEnablesWAF()



from __future__ import annotations

from typing import TYPE_CHECKING

from checkov.helm.image_referencer.base_provider import BaseHelmProvider
from checkov.kubernetes.image_referencer.provider.k8s import SUPPORTED_K8S_IMAGE_RESOURCE_TYPES

if TYPE_CHECKING:
    from igraph import Graph
    from networkx import DiGraph


class HelmProvider(BaseHelmProvider):
    def __init__(self, graph_connector: Graph | DiGraph, original_root_dir: str, temp_root_dir: str):
        super().__init__(
            graph_connector=graph_connector,
            supported_resource_types=SUPPORTED_K8S_IMAGE_RESOURCE_TYPES,
            original_root_dir=original_root_dir,
            temp_root_dir=temp_root_dir
        )



from typing import Optional, Any, Dict

from checkov.common.checks_infra.solvers.attribute_solvers.subset_attribute_solver import SubsetAttributeSolver
from checkov.common.graph.checks_infra.enums import Operators


class NotSubsetAttributeSolver(SubsetAttributeSolver):
    operator = Operators.NOT_SUBSET  # noqa: CCE003  # a static attribute

    def _get_operation(self, vertex: Dict[str, Any], attribute: Optional[str]) -> bool:
        return not super()._get_operation(vertex, attribute)



from __future__ import annotations

from typing import TYPE_CHECKING, Any

from checkov.cloudformation.image_referencer.base_provider import BaseCloudFormationProvider
from checkov.common.util.data_structures_utils import find_in_dict
from checkov.common.util.type_forcers import extract_json

if TYPE_CHECKING:
    from networkx import DiGraph


class AwsCloudFormationProvider(BaseCloudFormationProvider):
    def __init__(self, graph_connector: DiGraph) -> None:
        super().__init__(
            graph_connector=graph_connector,
            supported_resource_types=SUPPORTED_AWS_IMAGE_RESOURCE_TYPES,
        )


def extract_images_from_aws_apprunner_service(resource: dict[str, Any]) -> list[str]:
    image_names: list[str] = []

    image_repo = find_in_dict(input_dict=resource, key_path="SourceConfiguration/ImageRepository")
    if isinstance(image_repo, dict):
        repo_type = image_repo.get("ImageRepositoryType")
        name = image_repo.get("ImageIdentifier")
        if name and isinstance(name, str) and repo_type == "ECR_PUBLIC":
            image_names.append(name)

    return image_names


def extract_images_from_aws_batch_job_definition(resource: dict[str, Any]) -> list[str]:
    image_names: list[str] = []

    properties = extract_json(resource.get("ContainerProperties"))
    if isinstance(properties, dict):
        name = properties.get("Image")
        if name and isinstance(name, str):
            image_names.append(name)

    node_range = find_in_dict(input_dict=resource, key_path="NodeProperties/NodeRangeProperties")
    if isinstance(node_range, list):
        for node in node_range:
            name = find_in_dict(input_dict=node, key_path="Container/Image")
            if name and isinstance(name, str):
                image_names.append(name)

    return image_names


def extract_images_from_aws_codebuild_project(resource: dict[str, Any]) -> list[str]:
    image_names: list[str] = []

    name = find_in_dict(input_dict=resource, key_path="Environment/Image")
    if name and isinstance(name, str):
        # AWS provided images have an internal identifier
        if not name.startswith("aws/codebuild/"):
            image_names.append(name)

    return image_names


def extract_images_from_aws_ecs_task_definition(resource: dict[str, Any]) -> list[str]:
    image_names: list[str] = []

    definitions = extract_json(resource.get("ContainerDefinitions"))
    if isinstance(definitions, list):
        for definition in definitions:
            name = definition.get("Image")
            if name and isinstance(name, str):
                image_names.append(name)

    return image_names


def extract_images_from_aws_lightsail_container_service_deployment_version(resource: dict[str, Any]) -> list[str]:
    image_names: list[str] = []

    containers = find_in_dict(input_dict=resource, key_path="ContainerServiceDeployment/Containers")
    if isinstance(containers, list):
        for container in containers:
            name = container.get("Image")
            if name and isinstance(name, str):
                image_names.append(name)

    return image_names


# needs to be at the bottom to add the defined functions
SUPPORTED_AWS_IMAGE_RESOURCE_TYPES = {
    "AWS::AppRunner::Service": extract_images_from_aws_apprunner_service,
    "AWS::Batch::JobDefinition": extract_images_from_aws_batch_job_definition,
    "AWS::CodeBuild::Project": extract_images_from_aws_codebuild_project,
    "AWS::ECS::TaskDefinition": extract_images_from_aws_ecs_task_definition,
    "AWS::Lightsail::Container": extract_images_from_aws_lightsail_container_service_deployment_version,
}



from typing import Dict, Any

from checkov.common.models.enums import CheckResult
from checkov.kubernetes.checks.resource.base_container_check import BaseK8sContainerCheck


class AllowedCapabilities(BaseK8sContainerCheck):
    def __init__(self) -> None:
        # CIS-1.5 5.2.8
        name = "Minimize the admission of containers with added capability"
        # Do not generally permit containers with capabilities assigned beyond the default set
        # https://kubernetes.io/docs/concepts/policy/pod-security-policy/#capabilities
        # Location: container .securityContext.capabilities
        id = "CKV_K8S_25"
        super().__init__(name=name, id=id)

    def scan_container_conf(self, metadata: Dict[str, Any], conf: Dict[str, Any]) -> CheckResult:
        self.evaluated_container_keys = ["securityContext/capabilities/add"]
        if conf.get("securityContext"):
            if conf["securityContext"].get("capabilities"):
                if conf["securityContext"]["capabilities"].get("add"):
                    return CheckResult.FAILED
        return CheckResult.PASSED


check = AllowedCapabilities()



from checkov.common.models.enums import CheckCategories
from checkov.arm.base_resource_value_check import BaseResourceValueCheck


class KeyBackedByHSM(BaseResourceValueCheck):
    def __init__(self):
        name = "Ensure that key vault key is backed by HSM"
        id = "CKV_AZURE_112"
        supported_resources = ['Microsoft.KeyVault/vaults/keys']
        categories = [CheckCategories.BACKUP_AND_RECOVERY]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self):
        return 'properties/kty'

    def get_expected_value(self):
        return 'RSA-HSM'

    def get_expected_values(self):
        return [self.get_expected_value(), 'EC-HSM']


check = KeyBackedByHSM()



from __future__ import annotations
from dataclasses import dataclass
from typing import Any


@dataclass
class ReportCVE:
    cveId: str
    status: str
    severity: str
    packageName: str
    packageVersion: str
    link: str | None
    publishedDate: str
    cvss: int | None
    vector: str | None
    description: str | None
    riskFactors: Any | None



from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck


class KeyVaultDisablesPublicNetworkAccess(BaseResourceValueCheck):
    def __init__(self):
        name = "Ensure that Azure Key Vault disables public network access"
        id = "CKV_AZURE_189"
        supported_resources = ('azurerm_key_vault',)
        categories = (CheckCategories.NETWORKING,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self):
        return "public_network_access_enabled"

    def get_expected_value(self):
        return False

    def scan_resource_conf(self, conf) -> CheckResult:
        """
        KeyVaultDisablesPublicNetworkAccess unique logic.
        public_network_access_enabled default value is True (when creating it)
        If it False - check pass
        Otherwise, we check that ip rules configured inside network_acls.

        """
        conf_value = conf.get(self.get_inspected_key())
        conf_value = conf_value[0] if isinstance(conf_value, list) else conf_value
        if self.get_expected_value() == conf_value:
            return CheckResult.PASSED
        if conf.get("network_acls"):
            network_acls = conf.get("network_acls")
            if isinstance(network_acls, list):
                for network_acl in network_acls:
                    if isinstance(network_acl, dict):
                        ip_rules = network_acl.get("ip_rules")
                        # Get first element in ip_rules (as parser wrap it with list).
                        ip_rules = ip_rules[0] if ip_rules and isinstance(ip_rules, list) else ip_rules
                        if ip_rules:
                            return CheckResult.PASSED

        return CheckResult.FAILED


check = KeyVaultDisablesPublicNetworkAccess()



from typing import Optional, Any, Dict

from checkov.common.checks_infra.solvers.attribute_solvers.base_number_of_words_attribute_solver import \
    BaseNumberOfWordsAttributeSolver
from checkov.common.graph.checks_infra.enums import Operators


class NumberOfWordsEqualsAttributeSolver(BaseNumberOfWordsAttributeSolver):
    operator = Operators.NUMBER_OF_WORDS_EQUALS  # noqa: CCE003  # a static attribute

    def _get_operation(self, vertex: Dict[str, Any], attribute: Optional[str]) -> bool:
        attr = vertex.get(attribute)  # type:ignore[arg-type]  # due to attribute can be None

        if not self._validate_vertex_value(attr):
            return False

        num_of_words = self._get_number_of_words(attr)
        value_numeric = self._numerize_value()

        return num_of_words == value_numeric



from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_negative_value_check import BaseResourceNegativeValueCheck


class ALBACLIsUnrestricted(BaseResourceNegativeValueCheck):
    def __init__(self):
        name = "Alibaba ALB ACL does not restrict Access"
        id = "CKV_ALI_29"
        supported_resources = ['alicloud_alb_acl_entry_attachment']
        categories = [CheckCategories.NETWORKING]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self):
        return 'entry'

    def get_forbidden_values(self):
        return ["0.0.0.0/0"]


check = ALBACLIsUnrestricted()



from checkov.bicep.checks.param.base_param_check import BaseParamCheck, CheckovParameterAttributes
from checkov.common.models.enums import CheckResult, CheckCategories

# https://docs.microsoft.com/en-us/azure/azure-resource-manager/templates/test-cases#secure-parameters-cant-have-hardcoded-default


class SecureStringParameterNoHardcodedValue(BaseParamCheck):
    def __init__(self) -> None:
        name = "SecureString parameter should not have hardcoded default values"
        id = "CKV_AZURE_131"
        supported_type = ("string",)
        categories = (CheckCategories.SECRETS,)
        super().__init__(name=name, id=id, categories=categories, supported_type=supported_type)

    def scan_param_conf(self, conf: CheckovParameterAttributes) -> CheckResult:
        if not any(decorator["type"] == "secure" for decorator in conf["decorators"]):
            # if the decorator '@secure()' is not set, then it is a normal string
            return CheckResult.UNKNOWN
        default_value = conf.get("default")
        if default_value:  # should be missing, or an empty string
            conf["CKV_AZURE_131_secret"] = str(default_value)
            return CheckResult.FAILED
        else:
            return CheckResult.PASSED


check = SecureStringParameterNoHardcodedValue()



from __future__ import annotations

from typing import Any

from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_check import BaseResourceCheck


class SecurityGroupsIngressStatelessSecurityRules(BaseResourceCheck):
    def __init__(self) -> None:
        name = "Ensure security group has stateless ingress security rules"
        id = "CKV_OCI_21"
        supported_resources = ('oci_core_network_security_group_security_rule',)
        categories = (CheckCategories.NETWORKING,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: dict[str, list[Any]]) -> CheckResult:
        stateless = conf.get('stateless')
        direction = conf.get('direction')
        self.evaluated_keys = ["direction"]
        if direction and direction[0] == 'INGRESS':
            self.evaluated_keys.append("stateless")
            if stateless is None or stateless[0] is False:
                return CheckResult.FAILED
            return CheckResult.PASSED
        return CheckResult.UNKNOWN


check = SecurityGroupsIngressStatelessSecurityRules()



from typing import Any

from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck
from checkov.common.models.enums import CheckCategories
from checkov.common.models.consts import ANY_VALUE


class NetworkFirewallPolicyDefinesCMK(BaseResourceValueCheck):
    def __init__(self) -> None:
        name = "Ensure Network Firewall Policy defines an encryption configuration that uses a " \
               "customer managed Key (CMK)"
        id = "CKV_AWS_346"
        supported_resources = ("aws_networkfirewall_firewall_policy",)
        categories = (CheckCategories.ENCRYPTION,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        return "encryption_configuration/[0]/key_id"

    def get_expected_value(self) -> Any:
        return ANY_VALUE


check = NetworkFirewallPolicyDefinesCMK()



from __future__ import annotations

from typing import List, Union, Dict, Any, TYPE_CHECKING

from checkov.terraform.checks.data.base_cloudsplaining_data_iam_check import BaseTerraformCloudsplainingDataIAMCheck

if TYPE_CHECKING:
    from cloudsplaining.scan.policy_document import PolicyDocument


class CloudSplainingCredentialsExposure(BaseTerraformCloudsplainingDataIAMCheck):
    excluded_actions = {"ecr:GetAuthorizationToken"}  # noqa: CCE003  # a static attribute

    def __init__(self) -> None:
        name = "Ensure IAM policies does not allow credentials exposure"
        id = "CKV_AWS_107"
        super().__init__(name=name, id=id)

    def cloudsplaining_analysis(self, policy: PolicyDocument) -> Union[List[str], List[Dict[str, Any]]]:
        return [x for x in policy.credentials_exposure if x not in CloudSplainingCredentialsExposure.excluded_actions]


check = CloudSplainingCredentialsExposure()



from typing import List, Any

from checkov.cloudformation.checks.resource.base_resource_negative_value_check import BaseResourceNegativeValueCheck
from checkov.common.models.enums import CheckCategories


class S3PublicACLWrite(BaseResourceNegativeValueCheck):
    def __init__(self) -> None:
        name = "Ensure the S3 bucket does not allow WRITE permissions to everyone"
        id = "CKV_AWS_57"
        supported_resources = ("AWS::S3::Bucket",)
        categories = (CheckCategories.LOGGING,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_forbidden_values(self) -> List[Any]:
        return ["PublicReadWrite"]

    def get_inspected_key(self) -> str:
        return "Properties/AccessControl"


check = S3PublicACLWrite()



from checkov.common.models.enums import CheckCategories
from checkov.cloudformation.checks.resource.base_resource_value_check import BaseResourceValueCheck


class ElasticsearchDomainAuditLogging(BaseResourceValueCheck):
    def __init__(self) -> None:
        name = "Ensure Elasticsearch Domain Audit Logging is enabled"
        id = "CKV_AWS_317"
        supported_resources = ("AWS::Elasticsearch::Domain", "AWS::OpenSearchService::Domain")
        categories = (CheckCategories.LOGGING,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        return "Properties/LogPublishingOptions/AUDIT_LOGS/Enabled"


check = ElasticsearchDomainAuditLogging()



from __future__ import annotations

from dataclasses import dataclass
from typing import cast, Literal


@dataclass
class BlockType:
    RESOURCE: Literal["resource"] = "resource"
    MODULE: Literal["module"] = "module"

    def get(self, attr_name: str) -> str:
        return cast("str", getattr(self, attr_name.upper()))



from __future__ import annotations
from typing import Any

from checkov.circleci_pipelines.image_referencer.provider import CircleCIProvider
from checkov.common.images.workflow.image_referencer_manager import WorkflowImageReferencerManager


class CircleCIImageReferencerManager(WorkflowImageReferencerManager):

    def __init__(self, workflow_config: dict[str, Any], file_path: str) -> None:
        provider = CircleCIProvider(workflow_config=workflow_config, file_path=file_path)
        super().__init__(workflow_config, file_path, provider)



from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceCheck

ACCEPTED_VALUES = (
    "enabled",  # Legacy value, but still valid
    "ddl",  # Equivalent to the legacy value enabled
    "all",
)


class DocDBAuditLogs(BaseResourceCheck):
    def __init__(self):
        name = "Ensure DocDB has audit logs enabled"
        id = "CKV_AWS_104"
        supported_resources = ("aws_docdb_cluster_parameter_group",)
        categories = (CheckCategories.LOGGING,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf):
        self.evaluated_keys = ["parameter"]

        if "parameter" in conf:
            for idx, elem in enumerate(conf["parameter"]):
                if isinstance(elem, dict) and elem["name"][0] == "audit_logs":
                    if any(v in elem["value"][0] for v in ACCEPTED_VALUES):
                        self.evaluated_keys = [f"parameter/[{idx}]/name", f"parameter/[{idx}]/value"]
                        return CheckResult.PASSED
        return CheckResult.FAILED


check = DocDBAuditLogs()



from __future__ import annotations

from typing import Any

from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceValueCheck


class AzureFirewallDenyThreatIntelMode(BaseResourceValueCheck):
    def __init__(self) -> None:
        """
        https://azure.github.io/PSRule.Rules.Azure/en/rules/Azure.Firewall.Mode/
        Configure deny on threat intel for classic managed Azure Firewalls
        """
        name = "Ensure DenyIntelMode is set to Deny for Azure Firewalls"
        id = "CKV_AZURE_216"
        supported_resources = ("azurerm_firewall",)
        categories = (CheckCategories.NETWORKING,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        return 'threat_intel_mode'

    def get_expected_value(self) -> Any:
        return "Deny"


check = AzureFirewallDenyThreatIntelMode()



from __future__ import annotations

from typing import Any

from checkov.common.models.enums import CheckCategories, CheckResult
from checkov.arm.base_resource_check import BaseResourceCheck


class DataFactoryUsesGitRepository(BaseResourceCheck):
    def __init__(self) -> None:
        name = "Ensure that Azure Data Factory uses Git repository for source control"
        id = "CKV_AZURE_103"
        supported_resources = ("Microsoft.DataFactory/factories",)
        categories = (CheckCategories.GENERAL_SECURITY,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: dict[str, Any]) -> CheckResult:
        properties = conf.get("properties")
        if properties and isinstance(properties, dict):
            self.evaluated_keys = ["properties/repoConfiguration/type"]
            repo = properties.get("repoConfiguration")
            if not repo:
                return CheckResult.FAILED
            if repo and isinstance(repo, dict) and repo.get("type") is not None:
                return CheckResult.PASSED
            return CheckResult.UNKNOWN
        return CheckResult.FAILED


check = DataFactoryUsesGitRepository()



from typing import Dict, List, Any

from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.common.util.type_forcers import force_list
from checkov.terraform.checks.data.base_check import BaseDataCheck


class IAMPublicActionsPolicy(BaseDataCheck):
    def __init__(self):
        name = 'Ensure no IAM policies documents allow ALL or any AWS principal permissions to the resource'
        id = "CKV_AWS_283"
        supported_data = ["aws_iam_policy_document"]
        categories = [CheckCategories.IAM]
        super().__init__(name=name, id=id, categories=categories, supported_data=supported_data)

    def scan_data_conf(self, conf: Dict[str, List[Any]], entity_type: str) -> CheckResult:
        statements = force_list(conf.get('statement'))
        for statement in statements:
            if isinstance(statement, dict):
                if not statement.get('condition'):
                    # * on Deny is fine
                    if not statement.get('effect') == ["Deny"]:
                        principals = force_list(statement.get('principals'))
                        for principal in principals:
                            if isinstance(principal, dict):
                                principal_type = principal.get('type', [''])[0]
                                principal_identifiers = principal.get('identifiers', [])
                                if principal_type == 'AWS' and principal_identifiers and isinstance(principal_identifiers[0], list) and '*' in principal_identifiers[0]:
                                    return CheckResult.FAILED

        return CheckResult.PASSED


check = IAMPublicActionsPolicy()



from typing import Any

from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_negative_value_check import BaseResourceNegativeValueCheck


class GKEClusterLogging(BaseResourceNegativeValueCheck):
    def __init__(self) -> None:
        name = "Ensure Stackdriver Logging is set to Enabled on Kubernetes Engine Clusters"
        id = "CKV_GCP_1"
        supported_resources = ("google_container_cluster",)
        categories = (CheckCategories.KUBERNETES,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        return "logging_service"

    def get_forbidden_values(self) -> Any:
        return "none"


check = GKEClusterLogging()



from typing import List, Dict, Any

from checkov.common.models.enums import CheckResult, CheckCategories
from checkov.common.util.secrets import string_has_secrets
from checkov.terraform.checks.resource.base_resource_value_check import BaseResourceCheck


class VMCredsInCustomData(BaseResourceCheck):
    def __init__(self) -> None:
        name = "Ensure that no sensitive credentials are exposed in VM custom_data"
        id = "CKV_AZURE_45"
        supported_resources = ("azurerm_virtual_machine",)
        categories = (CheckCategories.SECRETS,)
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def scan_resource_conf(self, conf: Dict[str, List[Any]]) -> CheckResult:
        os_profile = conf.get("os_profile")
        if os_profile:
            os_profile = os_profile[0]
            if isinstance(os_profile, dict):
                custom_data = os_profile.get("custom_data")
                if custom_data:
                    custom_data = custom_data[0]
                    if isinstance(custom_data, str):
                        if string_has_secrets(custom_data):
                            conf[f'{self.id}_secret'] = custom_data
                            return CheckResult.FAILED
        return CheckResult.PASSED

    def get_evaluated_keys(self) -> List[str]:
        return ["os_profile/[0]/custom_data"]


check = VMCredsInCustomData()



from __future__ import annotations

import itertools
import os
import typing
from abc import abstractmethod
from typing import TYPE_CHECKING, Callable, Any, Mapping, Generator

from checkov.common.graph.graph_builder import CustomAttributes
from checkov.common.images.image_referencer import Image
from checkov.common.typing import LibraryGraph

if TYPE_CHECKING:
    import networkx
    import igraph
    import rustworkx
    from typing_extensions import TypeAlias

_ExtractImagesCallableAlias: TypeAlias = Callable[["dict[str, Any]"], "list[str]"]


class GraphImageReferencerProvider:
    __slots__ = ("graph_connector", "supported_resource_types", "graph_framework")

    def __init__(self, graph_connector: LibraryGraph,
                 supported_resource_types: dict[str, _ExtractImagesCallableAlias] | Mapping[
                     str, _ExtractImagesCallableAlias]):
        self.graph_connector = graph_connector
        self.supported_resource_types = supported_resource_types
        self.graph_framework = os.environ.get('CHECKOV_GRAPH_FRAMEWORK', 'IGRAPH')

    @abstractmethod
    def extract_images_from_resources(self) -> list[Image]:
        pass

    def extract_nodes(self) -> LibraryGraph | None:
        if self.graph_framework == 'IGRAPH':
            return self.extract_nodes_igraph()
        elif self.graph_framework == 'RUSTWORKX':
            return self.extract_nodes_rustworkx()
        else:
            return self.extract_nodes_networkx()

    def extract_nodes_networkx(self) -> networkx.Graph:
        if typing.TYPE_CHECKING:
            self.graph_connector = typing.cast(networkx.Graph, self.graph_connector)
        resource_nodes = [
            node
            for node, resource_type in self.graph_connector.nodes(data=CustomAttributes.RESOURCE_TYPE)
            if resource_type and resource_type in self.supported_resource_types
        ]

        return self.graph_connector.subgraph(resource_nodes)

    def extract_nodes_rustworkx(self) -> rustworkx.PyDiGraph[Any, Any]:
        resource_nodes = [
            index
            for index, node in self.graph_connector.nodes()
            if self.resource_type_pred(node, list(self.supported_resource_types))
        ]

        return self.graph_connector.subgraph(resource_nodes)

    def extract_nodes_igraph(self) -> igraph.Graph:
        if typing.TYPE_CHECKING:
            self.graph_connector = typing.cast(igraph.Graph, self.graph_connector)
        resource_nodes = [
            node
            for node, resource_type in itertools.zip_longest(self.graph_connector.vs['name'],
                                                             self.graph_connector.vs[CustomAttributes.RESOURCE_TYPE])
            if resource_type and resource_type in self.supported_resource_types
        ]
        return self.graph_connector.subgraph(resource_nodes)

    def extract_resource(self, supported_resources_graph: LibraryGraph) -> \
            Generator[dict[str, Any], dict[str, Any], dict[str, Any]]:
        def extract_resource_networkx(graph: networkx.Graph) -> Generator[dict[str, Any], None, None]:
            for _, resource in graph.nodes(data=True):
                yield resource

        def extract_resource_rustworkx(graph: rustworkx.PyDiGraph[Any, Any]) -> Generator[dict[str, Any], None, None]:
            for _, resource in graph.nodes():
                yield resource

        def extract_resource_igraph(graph: igraph.Graph) -> Generator[dict[str, Any], None, None]:
            for v in graph.vs:
                resource = {
                    'name': v['name'],
                    'block_type_': v[CustomAttributes.BLOCK_TYPE],
                    'resource_type': v[CustomAttributes.RESOURCE_TYPE]
                }
                resource.update(v['attr'])
                yield resource

        graph_resource = None
        if self.graph_framework == 'NETWORKX':
            graph_resource = extract_resource_networkx(supported_resources_graph)
        elif self.graph_framework == 'IGRAPH':
            graph_resource = extract_resource_igraph(supported_resources_graph)
        elif self.graph_framework == 'RUSTWORKX':
            graph_resource = extract_resource_rustworkx(supported_resources_graph)

        return graph_resource  # type: ignore

    @staticmethod
    def resource_type_pred(v: dict[str, Any], resource_types: list[str]) -> bool:
        return not resource_types or ("resource_type" in v and v["resource_type"] in resource_types)



from checkov.terraform.checks.resource.base_resource_negative_value_check import BaseResourceNegativeValueCheck
from checkov.common.models.enums import CheckCategories, CheckResult


class EFSAccessPointRoot(BaseResourceNegativeValueCheck):
    def __init__(self):
        """
        NIST.800-53.r5 AC-6(10)
        EFS access points should enforce a root directory (avoiding root / default)
        """
        name = "EFS access points should enforce a root directory"
        id = "CKV_AWS_329"
        supported_resources = ['aws_efs_access_point']
        categories = [CheckCategories.GENERAL_SECURITY]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources,
                         missing_attribute_result=CheckResult.FAILED)

    def get_inspected_key(self):
        return "root_directory/[0]/path"

    def get_forbidden_values(self):
        return "/"


check = EFSAccessPointRoot()



from __future__ import annotations

import logging
import os
from collections.abc import Collection
from pathlib import Path
from typing import TYPE_CHECKING

from pycep import BicepParser

from checkov.common.util.file_utils import read_file_with_any_encoding

if TYPE_CHECKING:
    from pycep.typing import BicepJson


class Parser:
    def __init__(self) -> None:
        self.bicep_parser = BicepParser(add_line_numbers=True)

    def parse(self, file_path: Path) -> tuple[BicepJson, list[tuple[int, str]]] | tuple[None, None]:
        try:
            content = read_file_with_any_encoding(file_path=file_path)
            template = self.bicep_parser.parse(text=content)
        except Exception:
            logging.debug(f"[bicep] Couldn't parse {file_path}", exc_info=True)
            return None, None

        file_lines = [(idx + 1, line) for idx, line in enumerate(content.splitlines(keepends=True))]

        return template, file_lines

    def get_files_definitions(
        self, file_paths: "Collection[Path]"
    ) -> tuple[dict[Path, BicepJson], dict[Path, list[tuple[int, str]]], list[str]]:
        logging.info(f"[bicep] start to parse {len(file_paths)} files")

        definitions: dict[Path, BicepJson] = {}
        definitions_raw: dict[Path, list[tuple[int, str]]] = {}
        parsing_errors: list[str] = []

        for file_path in file_paths:
            template, file_lines = self.parse(file_path)
            if template and file_lines:
                definitions[file_path] = template
                definitions_raw[file_path] = file_lines
            else:
                parsing_errors.append(os.path.normpath(file_path.absolute()))

        logging.info(f"[bicep] successfully parsed {len(definitions)} files")

        return definitions, definitions_raw, parsing_errors



from typing import Optional, Any, Dict

from checkov.common.graph.checks_infra.enums import Operators
from checkov.common.checks_infra.solvers.attribute_solvers.base_attribute_solver import BaseAttributeSolver


class EqualsIgnoreCaseAttributeSolver(BaseAttributeSolver):
    operator = Operators.EQUALS_IGNORE_CASE  # noqa: CCE003  # a static attribute

    def _get_operation(self, vertex: Dict[str, Any], attribute: Optional[str]) -> bool:
        attr_val = vertex.get(attribute)  # type:ignore[arg-type]  # due to attribute can be None
        # if this value contains an underendered variable, then we cannot evaluate the check,
        # so return True (since we cannot return UNKNOWN)
        # handle edge cases in some policies that explicitly look for blank values
        if self.value != '' and self._is_variable_dependant(attr_val, vertex['source_']):
            return True
        return str(attr_val).lower() == str(self.value).lower()



from __future__ import annotations

import logging
from typing import TYPE_CHECKING, List, cast

from checkov.common.bridgecrew.code_categories import CodeCategoryMapping, CodeCategoryType
from checkov.common.bridgecrew.integration_features.base_integration_feature import BaseIntegrationFeature
from checkov.common.bridgecrew.licensing import CategoryToSubscriptionMapping, CustomerSubscription, \
    open_source_categories
from checkov.common.bridgecrew.platform_integration import bc_integration

if TYPE_CHECKING:
    from checkov.common.bridgecrew.platform_integration import BcPlatformIntegration
    from checkov.common.output.report import Report
    from checkov.common.typing import _BaseRunner


LICENSE_KEY = 'platformLicense'
MODULES_KEY = 'modules'


class LicensingIntegration(BaseIntegrationFeature):
    def __init__(self, bc_integration: BcPlatformIntegration) -> None:
        super().__init__(bc_integration=bc_integration, order=6)
        self.enabled_modules: List[CustomerSubscription] = []
        self.open_source_only: bool = True

    @property
    def billing_plan(self) -> None:
        # Deprecated, already calculated in the BE into the enabled_modules list
        return None

    def is_valid(self) -> bool:
        # We will always use this integration to determine what runs or not
        return True

    def pre_scan(self) -> None:
        if not self.bc_integration.bc_api_key:
            logging.debug('Running without API key, so only open source runners will be enabled')
            self.open_source_only = True
        elif not self.bc_integration.customer_run_config_response:
            logging.debug('Customer run config response does not exist, but there is an API key, so there may be some integration issue. Proceeding with open source runners.')
            self.open_source_only = True
        else:
            logging.debug('Found customer run config and using it for licensing')
            license_details = self.bc_integration.customer_run_config_response.get(LICENSE_KEY)
            logging.debug(f'User license details: {license_details}')

            self.open_source_only = False
            # the API will return True for all modules if they are on resource mode, so we don't actually need the billing plan explicitly here
            self.enabled_modules = [CustomerSubscription(m) for m, e in license_details.get(MODULES_KEY).items() if e]

    def is_runner_valid(self, runner_check_type: str) -> bool:
        logging.debug(f'Checking if {runner_check_type} is valid for license')
        if self.open_source_only:
            enabled = CodeCategoryMapping[runner_check_type] in open_source_categories  # new secrets are disabled, but the runner is valid
            logging.debug(f'Open source mode - the runner is {"en" if enabled else "dis"}abled')
        else:
            sub_type = LicensingIntegration.get_subscription_for_runner(runner_check_type)
            enabled = sub_type in self.enabled_modules
            logging.debug(f'Customer mode - the {sub_type} subscription is {"en" if enabled else "dis"}abled')

        return enabled

    def should_run_image_referencer(self) -> bool:
        return not self.open_source_only and CustomerSubscription.SCA in self.enabled_modules

    @staticmethod
    def get_subscription_for_runner(runner_check_type: str) -> CustomerSubscription:
        if 'sca_' in runner_check_type:
            # SCA runners currently have two CodeCategoryTypes
            return CustomerSubscription.SCA
        else:
            return CategoryToSubscriptionMapping[cast(CodeCategoryType, CodeCategoryMapping[runner_check_type])]

    def post_runner(self, scan_report: Report) -> None:
        pass

    def pre_runner(self, runner: _BaseRunner) -> None:
        pass

    def post_scan(self, merged_reports: list[Report]) -> None:
        # not used
        pass


integration = LicensingIntegration(bc_integration)
